/* Documentation of the STREAM module.  This file is not compiled. 
 */

/* An object of type Stream is really a file descriptor.  Open() opens a
   named file but a Stream can also be opened as another type of file
   using functions of other modules. 
 */

class Stream
{
public:

	enum {read, write, append, write_excl,
	      binary, text}; 

//	enum Type {dir, regular, pipe, link, block, character, socket};

	enum {p_read, p_write, p_execute, p_extra};

	Stream();
	/* Stream is closed on creation.
	 */
	~Stream();
	/* Stream is closed automatically. 
	 */

	bool Is() const;
	bool Open(String file_name, 
		  int open_mode, 
		  int type= binary);
	/* Opens the file with name FILE_NAME.  The Stream object must be
	   closed. 

	   OPEN_MODE=
  	       read:   Open an existing file for reading.  Fail if the
 	               file does not exist. 
	       write:  Open a file for writing.  Remove an existing
	               file when necessary. 
	       append: Open a file for appending.  Create the file if
	               it does not exist. 
	       write_excl:  Open the file which must not exist.  Fail
	                    if the file exists. 

	   TYPE=
	   text:   open as text file
	   binary: open as binary file

	   Return value: Wether opening was successfull.
	*/

	/* In the following non-static functions, THIS must be open. 
	 */

	bool Close();
	/* Return FALSE on error.  If an error occurs, This will be
	   closed, but data was not written or read completely. 
	 */
	int Read(void *data, int length);
	/* Read LENGTH bytes and store them in DATA.  Return the
	   actual number of byte read.  Thus the return value is
	   LENGTH on success, and less on error or end-of-file. 
	 */
	bool Write(const void *data, int length);
	/* Write LENGTH byte from DATA.  Return FALSE on error. 
	 */
	bool End_Of_File() const;

	bool Read_Line(char *data, 
		       int   length);
	/* Read one line and write it into DATA.  The line contains
	   the newline, and is terminated by '\0'.  On error, FALSE is
	   returned and DATA is invalid.  Else DATA is filled with
	   almost LENGTH bytes.  DATA will always be terminated with
	   '\0' even when the line would have been longer.  You can
	   find out that a line was not read completely when the
	   string in DATA is not newline-terminated. 
	 */

	bool Write_Line(const char *s);
	/* Write the string.  Return TRUE on success.  S must not be NULL. 
	 */
	bool Write_String(String text);
	char Read_Char();
	/* Read a character.  Return '\0' on end of file or error. 
	 */
	bool Write_Char(char);
	/* return value:  success
	 */
	void Flush();

	void Set_Block();
	/* Set what corresponds to the negation of the O_NONBLOCK
	   flag.  Default is TRUE.  If FALSE, a read returns an error
	   instead of blocking.  
	*/
#if 0
	bool Get_Stat(Stat &) const;
	/* Equivalent to the static function of the same name. 
	 */
	
	static bool Get_Stat(String  file_name, 
			     Stat   &stat, 
			     bool    follow_link= true);
	/* Get information about the file FILE_NAME.  Write the information
	   in P_STAT.  Return FALSE on failure. 
	*/
#endif /* 0 */ 
	static bool Remove(String file_name);
	/* Remove file with file name _file_name.
	   Return Value: success
	*/
	/* _file_name must not be NULL. 
	 */
	static bool Access(String file_name, int _type);
	/* TRUE if access is possible according to Real UID.  TYPE must be p_% but not
	   p_extra. 
	*/ 
};
