#ifndef ASSERT_INCLUDED
#define ASSERT_INCLUDED

/* Check internal consistency.

   If the check fails, the function Assert_Failed() of the
   ASSERT_FAILED module is called.  

   The assertions return zero if they succeed, and non-zero if they
   fail.   
 */

#ifdef NDEBUG

#define Assert(test)       (0)
#define Assert_Unreached() ((void) 0)

#else /* ! NDEBUG */

#include "assert_failed.hh"

#define Assert(test) ((test) ? 0 : (Assert_Failed(__FILE__, __LINE__, \
                      # test, __PRETTY_FUNCTION__), 0))
#define Assert_Unreached() (Assert_Failed(__FILE__, __LINE__, \
		      "Unreacheable code", __PRETTY_FUNCTION__), 0)

#endif /* ! NDEBUG */

#endif /* ! ASSERT_INCLUDED */
