#include "board.hh"

#include "main.hh"
#include "text_utils.hh"
#include "math.hh"
#include "board_image.hh"
#include "trl.hh"
#include "format.hh"
#include "dbg.hh"

int Board::frame_width= -1;

bool
Board::Open(Pointer <Widget> parent, 
	    const Open_Info &open_info)
{
	dragging= false;

	msg_move= open_info.msg_move;
	msg_middle= open_info.msg_middle;
	msg_turn= open_info.msg_turn;

	persistent_settings.Open(This, msg_settings);

	position= open_info.position;
	square_width= -1;

	selection[0]= selection[1]= 0;

	have_colors= false;

	if (! Widget::Open(parent))
		return false;

	if (frame_width < 0)
		frame_width= Canvas::Get_Ascent() + (unit / 10 >? 2); 

	Change_Sized_Image();
//	background_invalid= true;

	return true;
}

void
Board::Wdg_Draw(Canvas &canvas)
{
	Dbg_Function();

	if (! (board_image.Is() && board_image.Has_Canvas()))
		return ;

	board_image().Set_Draw_Mode(Canvas::dm_copy);
	canvas.Draw_Canvas(board_image(),
			   Get_Size(),
			   Point(0));
}

void
Board::Change_Sized_Image()
{
	Dbg_Function();

	Interrupt_Dragging();

	int w= persistent_settings().flags & bs_frame ? 2 * frame_width : 0;
	int new_square_width= ((Get_Size().x <? Get_Size().y) - w) / 8 >? 1; 
	
	if (new_square_width != square_width)
	{
		square_width= new_square_width;
		Dbg_Int(persistent_settings().flags & bs_anti_alias);
		scaled_image.Load(square_width, 
				  persistent_settings().flags & bs_anti_alias);

		board_image.Free();
		Dbg_("creating board_image.");
		board_image.Create(Get_Size());
//		board_image.Create(Point(8 * square_width + w));

		if (board_image.Is())
		{
			Dbg_("\tsuccess.");
			Draw_All();
			Draw_Frame();
		}
		else
		{
			Dbg_("\tfailed.");
		}
		
		drag_img.Free();
//		drag_img.Create(Point(8 * square_width + w));
		drag_img.Create(Get_Size()); 
	}

	board_image().Set_Color(Canvas::Get_Color(RGB(0xea, 0xea,
						      0xea)));
	board_image().Set_Draw_Mode(Canvas::dm_copy); 
		
	int width= (persistent_settings().flags & bs_frame) ? 2 * frame_width : 0;
	width += 8 * square_width;
	Dbg_Int(width);
	board_image().Draw_Rect(Rect(width,
			      0, 
			      Get_Size().x - width,
			      width));
	board_image().Draw_Rect(Rect(0, 
			      width, 
			      Get_Size().x,
			      Get_Size().y - width));
}

void
Board::Draw_All()
{
	Dbg_Function_();

	for (int file= sq_a;  file <= sq_h; ++ file)
		for (int rank= sq_1;  rank <= sq_8;  ++ rank)
			Draw_Square(file, rank);
}

void Board::
Draw_Square(int file, 
	    int rank, 
	    bool always_empty)
{
	Dbg_Function_();

	if (! scaled_image.Is())
		return; 

	Assert (Is_Inside_Square(Make_Square(file, rank)));

	Rect rect= Square_Rect(file, rank);

	if (! (board_image.Is() && board_image.Has_Canvas()))
		return;

	Draw_Square_Background(board_image(), file, rank, rect);

	if (always_empty)
		return;

	Piece piece= position().Get_Piece(Make_Square(file, rank));

	if (piece.Is() && scaled_image.Get_Image().Has_Canvas())
	{
		board_image().Set_Draw_Mode(Canvas::dm_copy);

		scaled_image.Draw(board_image(),
				  rect.origin,
				  piece.Get_Type(),
				  piece.Get_Color());
	}
}

void
Board::Wdg_Size(Point)
{
	Dbg_Function_();
	Change_Sized_Image();
}

int
Board::Point_Square(Point p) 
	const
{
	if (persistent_settings().flags & bs_frame)
		p -= frame_width;

	int square= Make_Square(sq_a + p.x / square_width,
			      sq_8 - p.y / square_width);

	if (persistent_settings().flags & bs_turned)
		Square_Turn(square);

	return square;
}

bool
Board::Get_Squares(int &from, 
		   int &to)
{
	from= drag_from;
	to=   drag_to;

	return true;
}

void Board::Invalidate_Position()
{
	Dbg_Function_(); 
	Draw_All();
	Invalidate();
}

void Board::
Menued_Get(int menu,
	   Menued::Entries &entries)
{
	switch (menu)
	{
	default:  Assert_Unreached();  break;

	case menu_main:
		entries.Append_Checked(msg_turn_board,
				    Trl("Turn board"),
				    Get_Turned(),
				    false);
		entries.Append_Checked(msg_frame,
				    Trl("Frame"),
				    Get_Settings().flags & bs_frame,
				    false);
		entries.Append_Checked(msg_square_names,
				    Trl("Square names"),
				    Get_Settings().flags & bs_show_names,
				    false);
		entries.Append_Checked(msg_anti_alias,
				    Trl("Smooth chess pieces"),
				    Get_Settings().flags & bs_anti_alias,
				    false);
		entries.Append_Separator();
		entries.Set_Menued(persistent_settings);
		persistent_settings.Menued_Get
			(Persistent <Board_Settings, Board_Settings_Editor> ::menu_main,
			 entries);
		break;
	}
}

void 
Board::MR_Receive(int id)
{
	Dbg_Function();

	switch (id)
	{
	default:
		Parent_Receive_Message(id);  
		break;

	case msg_square_names:
		persistent_settings.Get_Change_Value().flags ^= bs_show_names;
		persistent_settings.Changed();
		Invalidate_Position();
		break;

	case msg_turn_board:
		Set_Turned(! Get_Turned());
		break;

	case msg_settings:
	{
		Dbg("\tmsg_settings_edited");
		Interrupt_Dragging();
		bool old_aa= persistent_settings.Get_Old_Value().flags & bs_anti_alias;
		Dbg_Int(old_aa);
		bool old_fr= persistent_settings.Get_Old_Value().flags & bs_frame;
		Dbg_Int(old_fr);
		RGB old_frame_color= persistent_settings.Get_Old_Value().colors[3];
//		settings= edit.Get_Value();
		Dbg_Int(persistent_settings().flags & bs_anti_alias);
		Dbg_Int(persistent_settings().flags & bs_frame);
//		edit.Have_Read();
		have_colors= false;
		bool redraw= (old_aa != (persistent_settings().flags & bs_anti_alias)  ||
		    old_fr != (persistent_settings().flags & bs_frame)       ||
			      (old_fr && (old_frame_color != persistent_settings().colors[3])));
		Dbg_Int(redraw);
		if (redraw)
		{
			square_width= -1;
			Change_Sized_Image(); /* Invalidates only the interior */ 
			Invalidate(); 
		}
		else
			Invalidate_Position();
		break;
	}

	case msg_anti_alias:
		Interrupt_Dragging();
		square_width= -1;
		persistent_settings.Get_Change_Value().flags ^= bs_anti_alias; 
		persistent_settings.Changed();
		Change_Sized_Image();
		Invalidate_Position();
		break;

	case msg_frame:
		Interrupt_Dragging();
		persistent_settings.Get_Change_Value().flags ^= bs_frame;
		persistent_settings.Changed(); 
		square_width= -1;
		Change_Sized_Image();
		break;
	}
}

void Board::
Draw_Square_Background(Canvas &canvas,
		       int file,
		       int rank,
		       Rect rect)
{
	/* An empty square consists of:

	   - The background, which is either black or white. 
	   - The selection:  If this square is selected, draw a circle
	   over the background.
	   - The square name:  If square names are enabled, draw the
	   square name.
	 */

	/* Draw background.
	 */
	Chess_Color chess_color= Square_Color(Make_Square(file, rank));

	if (! have_colors)
	{
		have_colors= true;

		for (int i= 0;  i < 3;  ++i)
			colors[i]= Canvas::Get_Color(persistent_settings().colors[i]);
	}

	Canvas::Color color= colors[chess_color]; 

	canvas.Set_Draw_Mode(Canvas::dm_copy);

	/* The "black" background color is grey by default.  When it
	   would have been black, draw diagonal stripes, as black
	   piece on a black background are not visible. 
	*/
	if (chess_color == cc_b && color == Canvas::Get_Black())
	{
		const int line_count= 8;

		canvas.Set_Color(Canvas::Get_White(),
				 true);
		canvas.Draw_Rect(rect);

		canvas.Set_Color(color, true);
		canvas.Set_Line_Mode
			(((rect.size.x + rect.size.y) / 13 / line_count)
			 >? 1);

		for (int i= 0; i < line_count; ++i)
		{
			canvas.Draw_Line(rect.origin
					 + Point(0, 
						 i * rect.size.y / line_count),
					 rect.origin
					 + Point(i * rect.size.x / line_count,
						 0));
			canvas.Draw_Line(rect.origin
					 + Point(i * rect.size.x / line_count,
						 rect.size.y),
					 rect.origin
					 + Point(rect.size.x,
						 i * rect.size.y / line_count));
		}

		color= Canvas::Get_White();
		/* Use white background when drawing the square name,
		   even on dark squares. 
		*/
	}
	else  /* Filled background */
	{
		canvas.Set_Color(color, true);
		canvas.Draw_Rect(rect);
	}

	/* Draw selection circle.
	 */
	if (Make_Square(file, rank) == selection[0]    ||
	    Make_Square(file, rank) == selection[1])
	{
		canvas.Set_Color(Canvas::Get_Color(persistent_settings().colors[2]),
				 true);
		canvas.Draw_Ellipse(rect);
	}

	/* Draw square name 
	 */
	if (persistent_settings().flags & bs_show_names)
	{
		canvas.Set_Color(Canvas::Get_Black(), true);
		canvas.Set_Color(Canvas::Get_Color_1(), false); // color0
		canvas.Set_Draw_Mode(Canvas::dm_and);

		canvas.Draw_Label(Point(rect.origin.x,
					rect.origin.y + rect.size.y - Canvas::Get_Descent()),
				  Format_Square(Make_Square(file, rank)), dt_text);
	}
}

void
Board::Wdg_Mouse_Cursor()
{
	Set_Mouse_Cursor(ms_hand);
}

void
Board::Wdg_Mouse(Mouse_Action ma, 
		 Mouse_Button mb, 
		 Point p,
		 bool ac)
{
	Dbg_Function_();
	Dbg_Int_(ma);
	Dbg_("\tp= (%4d, %4d)", p.x, p.y);

	if (ma == ma_down && mb == mb_right && ac)
		/* context */
	{
		Interrupt_Dragging();
		menu.Popup(/*p */);
	}
	else if (ma == ma_down && mb == mb_middle && ac)
		/* middle mouse button clicked */
	{
		if (! Is_Inside_Point(p))
			return;
		Interrupt_Dragging();
		drag_from= Point_Square(p);
		Parent_Send_Message(msg_middle);
	}
	else if (dragging)
	{
		if (ma == ma_move)
			/* move pointer while dragging */
		{
			Point new_org= p - pointer_pos;

			/* temporary image */ 
			static Server_Image i;
			static int i_size= -1;

			if (i_size != square_width)
			{
				if (i.Is())
					i.Free();
				i.Create(Point(square_width));
				i_size= square_width;
			}

			if (i.Is() 
			    && i.Has_Canvas()
			    && drag_img.Has_Canvas()
			    && board_image.Has_Canvas()
			    && scaled_image.Get_Image().Has_Canvas())
			{
				/* draw background on board on old
				   location */ 
				board_image().Set_Draw_Mode(Canvas::dm_copy);
				board_image().Draw_Canvas
					(drag_img(),
					 Rect(last_org, Point(square_width)),
					 last_org);

				/* draw image into temporary */
				i().Set_Draw_Mode(Canvas::dm_copy); 
				i().Draw_Canvas(board_image(),
						Rect(Point(square_width)),
						new_org);
				/* draw new piece on temporary */ 
				scaled_image.Draw(i(),
						  Point(0),
						  dragged_piece.Get_Type(),
						  dragged_piece.Get_Color());
				/* draw temporary on board */ 
				board_image().Draw_Canvas
					(i(),
					 Rect(new_org, Point(square_width)),
					 Point(0));
				Invalidate(Rect(last_org, Point(square_width)));
				Invalidate(Rect(new_org,  Point(square_width)));

				last_org= new_org;
			}
		}
		else if (mb == mb_left && (ma == ma_up || ma == ma_down) && ac)
			/* end dragging */
		{
			Dbg_("\tEnd dragging.");

			/* We use P to calculate the release square,
			   but the piece is still drawn at LAST_ORG. 
			*/
		  
			Release_Capture();
			dragging= false;

			/* Release square. 
			 */

			Point middle= p - pointer_pos + Point(square_width / 2);
			/* Center of the dragged piece. 
			 */

			if (persistent_settings().flags & bs_frame)
				middle -= frame_width;
			int file= sq_a + middle.x / square_width;
			int rank= sq_8 - middle.y / square_width;

			/* Square of release. 
			 */
			drag_to= Make_Invalid_Square(file, rank);
			if (persistent_settings().flags & bs_turned)
				Square_Turn(drag_to);
			
			/* Copy Portion obscured by piece back from drag_img to 
			   board_img.
			*/
			Rect rect(last_org, Point(square_width));
			if (rect.origin.x < 0)
			{
				rect.size.x += rect.origin.x;
				rect.origin.x= 0;
			}
			if (rect.origin.y < 0)
			{
				rect.size.y += rect.origin.y;
				rect.origin.y= 0;
			}
//			int overall_width= 8 * square_width;
//			if (settings.flags & bs_frame)
//				overall_width += 2 * frame_width;
			Point osize= board_image.Get_Size();
			if (rect.origin.x + rect.size.x > osize.x)
				rect.size.x= osize.x - rect.origin.x;
			if (rect.origin.y + rect.size.y > osize.y)
				rect.size.y= osize.y - rect.origin.y;
			board_image().Draw_Canvas(drag_img(), rect, rect.origin);

			if (Is_Inside_Square(drag_to))
			{
			}
			else
			{
				drag_to= 0;
			}

			Parent_Send_Message(msg_move);
		}
	}
	else
	{
		if (mb == mb_left && ma == ma_down && ac)
			/* begin dragging */
		{
			if (! drag_img.Is())
				return;
			if (! Is_Inside_Point(p))
				return;
			drag_from= Point_Square(p);

			if (! Is_Inside_Square(drag_from))
				return;
			dragged_piece= position().Get_Piece(drag_from);
			if (! dragged_piece.Is())
				/* Drag an empty square. 
				 */ 
			{
				drag_to= drag_from;
				Parent_Send_Message(msg_move);
				return;
			}
			Capture();
			if (drag_img.Has_Canvas())
			{
				Canvas &c_drag= drag_img();
				if (board_image.Has_Canvas())
				{
					Canvas &c_board= board_image();
					c_drag.Set_Draw_Mode(Canvas::dm_copy);
					c_drag.Draw_Canvas
						(c_board,
						 Rect(Get_Size()), 
//						 Rect(Point(8 *
//							    square_width + (settings.flags & bs_frame ? 2 * frame_width : 0))),
						 Point(0));
				}
				Draw_Square_Background(c_drag, 
						       Square_File(drag_from),
						       Square_Rank(drag_from),
						       Square_Rect(Square_File(drag_from),
								   Square_Rank(drag_from)));

			}
			int w= persistent_settings().flags & bs_frame ? frame_width : 0;
			last_org= Point((p.x - w) / square_width * square_width + w,
					(p.y - w) / square_width * square_width + w);
			pointer_pos= p - last_org;

			dragging= true;
		}
	}
}

void
Board::Interrupt_Dragging()
{
	if (! dragging)
		return;

	Release_Capture();
	dragging= false;
	Draw_All();
	Invalidate();
}

Rect
Board::Square_Rect(int file, int rank) const
{
	if (persistent_settings().flags & bs_turned)
	{
		file= sq_h - (file - sq_a);
		rank= sq_8 - (rank - sq_1);
	}

	int w= persistent_settings().flags & bs_frame ? frame_width : 0;
	return Rect(square_width * (file - sq_a) + w,
		    square_width * (sq_8 - rank) + w,
		    square_width, square_width);
}

void
Board::Wdg_Close()
{
	persistent_settings.Close(); 
	position= NULL;
	menu.Close(); 
	Widget::Wdg_Close();
}

void
Board::Set_Selection(int a, int b)
{
	Dbg_Function_();
	Dbg_("\ta=  %s", Format_Square(a)());
	Dbg_("\tb=  %s", Format_Square(b)());

	Interrupt_Dragging();

	if (selection[0] != 0)
	{
		Dbg_("\t[0] was= %s", Format_Square(selection[0])());
		int
			file= Square_File(selection[0]),
			rank= Square_Rank(selection[0]);
		selection[0]= 0;
		Draw_Square(file, rank);
		Rect rect= Square_Rect(file, rank);
		Invalidate(rect);
	}
	if (selection[1] != 0)
	{
		Dbg_("\t[1] was= %s", Format_Square(selection[1])());
		int
			file= Square_File(selection[1]),
			rank= Square_Rank(selection[1]);
		selection[1]= 0;
		Draw_Square(file, rank);
		Rect rect= Square_Rect(file, rank); 
		Invalidate(rect);
	}

	{
		selection[0]= a;
		Draw_Square(Square_File(a),
			    Square_Rank(a));
		Rect rect= Square_Rect(Square_File(a),
				       Square_Rank(a));
		Invalidate(rect);
	}

	{
		selection[1]= b;
		Draw_Square(Square_File(b),
			    Square_Rank(b));
		Rect rect= Square_Rect(Square_File(b),
				       Square_Rank(b));
		Invalidate(rect);
	}
}

void
Board::Clear_Selection()
{
	Interrupt_Dragging();
	if (selection[0] != 0)
	{
		Draw_Square(Square_File(selection[0]),
			    Square_Rank(selection[0]));
		selection[0]= 0;
		Invalidate();
	}
	if (selection[1] != 0)
	{
		Draw_Square(Square_File(selection[1]),
			    Square_Rank(selection[1]));
		selection[1]= 0;
		Invalidate();
	}
}

void
Board::Wdg_Get_Size_Request(int &, 
			    int &,
			    bool &resizeable)
{
	resizeable= true; 
}

Board::Board()
	:  menu(This, menu_main)
{
}

void
Board::Invalidate_Square(int square,
			 bool interrupt_dragging)
{
	Assert (Is_Inside_Square(square));

	Dbg_Function_();
	Dbg_("\tsquare= %d", square);

	if (interrupt_dragging)
		Interrupt_Dragging(); 

	Draw_Square(Square_File(square), Square_Rank(square));
  
	Rect rect= Square_Rect(Square_File(square), Square_Rank(square));
	Invalidate(rect);
}

void Board::Set_Turned(bool new_turned) 
{
	Interrupt_Dragging();
	if (new_turned)
		persistent_settings.Get_Change_Value().flags |= bs_turned;
	else
		persistent_settings.Get_Change_Value().flags &= ~bs_turned; 
	persistent_settings.Changed();
	square_width= -1;
	Change_Sized_Image();

//	Invalidate_Position();
//	Draw_Frame();
	Parent_Send_Message(msg_turn);
}

void Board::Draw_Frame()
{
	Dbg_Function();

	if (! (persistent_settings().flags & bs_frame))
		return;

	if (! board_image.Is())
		return;
	if (! board_image.Has_Canvas())
		return;

	Canvas &canvas= board_image.Get_Canvas();

	canvas.Set_Draw_Mode(Canvas::dm_copy);
	canvas.Set_Color(persistent_settings().colors[3]);
	canvas.Draw_Rect(Rect(0, 
			      0,
			      2 * frame_width + 8 * square_width,
			      frame_width));
	canvas.Draw_Rect(Rect(0,
			      frame_width + 8 * square_width,
			      2 * frame_width + 8 * square_width,
			      frame_width));
	canvas.Draw_Rect(Rect(0,
			      frame_width,
			      frame_width,
			      8 * square_width));
	canvas.Draw_Rect(Rect(frame_width + 8 * square_width,
			      frame_width,
			      frame_width,
			      8 * square_width));

	canvas.Set_Color(Canvas::Get_Black());

	int ascent= Canvas::Get_Ascent();

	for (int file= 0;  file < 8;  ++file)
	{
		String text(persistent_settings().flags & bs_turned 
			    ? 'H' - file
			    : 'A' + file);

		int width= Canvas::Get_Label_Width(text);

		int x= frame_width + ((2 * file + 1) * square_width - width) / 2;
		int y= (frame_width - ascent) / 2 + ascent;

		canvas.Draw_Label(Point(x, y),
				  text,
				  dt_text);

		y += frame_width + 8 * square_width; 
		canvas.Draw_Label(Point(x, y),
				  text,
				  dt_text);
	}
	for (int rank= 0;  rank < 8;  ++rank)
	{
		String text(persistent_settings().flags & bs_turned
			    ? '1' + rank
			    : '8' - rank);
		int width= Canvas::Get_Label_Width(text);
		int y= frame_width + rank * square_width + (square_width - ascent) / 2 + ascent; 
//		int y= frame_width + ((2 * rank + 1) * square_width - ascent) / 2;
		int x= (frame_width - width) / 2;
		canvas.Draw_Label(Point(x, y), text, dt_text);
		x += frame_width + 8 * square_width;
		canvas.Draw_Label(Point(x, y), text, dt_text);
	}
}

void Board::Invalidate_Move_Squares(const Move &move)
{
	int changed[4];
	move.Squares_Touched(changed);
	Invalidate_Square(changed[0]);
	Invalidate_Square(changed[1]);
	if (changed[2])
		Invalidate_Square(changed[2]);
	if (changed[3])
		Invalidate_Square(changed[3]);
}

bool Board::Is_Inside_Point(Point p) const
{
	int w= persistent_settings().flags & bs_frame ? frame_width : 0;
	return (p.x >= w && p.x < 8 * square_width + w  &&
		p.y >= w && p.y < 8 * square_width + w);
}
