#include "board_settings_editor.hh"

void
Board_Settings_Editor::Editor_Window_Set_Value(Pointer <const Board_Settings> value)
{
	if (value == NULL)
		settings.Reset();
	else
		settings= value();
}

const Board_Settings &
Board_Settings_Editor::Editor_Window_Get_Value() 
{
	if (Is())
		Retrieve_Value();
	return settings;
}

bool
Board_Settings_Editor::Object_Editor_Open(Pointer <Widget> parent)
{
	if (! main_packer.Open(parent))
		return false;

	if (! left_packer.Open(main_packer))
		return false;

	if (! turned_check.Open(left_packer,
				settings.flags & bs_turned,
				Trl("Turn board")))
		return false;
	if (! frame_check.Open(left_packer,
			       settings.flags & bs_frame,
			       Trl("Frame")))
		return false;
	if (! show_names_check.Open(left_packer,
				    settings.flags & bs_show_names,
				    Trl("Square names")))
		return false;
	if (! anti_alias_check.Open(left_packer,
			    settings.flags & bs_anti_alias,
			    Trl("Smooth chess pieces")))
		return false;

	if (! bevel.Open(main_packer, false))
		return false; 

	if (! right_packer.Open(main_packer))
		return false;


	if (! light_chooser.Open(right_packer, 
				 settings.colors[0],
				 Trl("Light squares")))
		return false;
	if (! dark_chooser.Open(right_packer, 
				settings.colors[1],
				Trl("Dark squares")))
		return false;
	if (! last_move_chooser.Open(right_packer,
				     settings.colors[2],
				     Trl("Last move")))
		return false;
	if (! frame_chooser.Open(right_packer,
				 settings.colors[3],
				 Trl("Frame")))
		return false;
	return true; 
}
    
int 
Board_Settings_Editor::Editor_Window_Use()
{
	return use_ok;
}

bool
Board_Settings_Editor::Editor_Window_New()
{
	settings.Reset(); 
	show_names_check.Set_Checked(settings.flags & bs_show_names); 
	turned_check.Set_Checked(settings.flags & bs_turned); 
	anti_alias_check.Set_Checked(settings.flags & bs_anti_alias);
	frame_check.Set_Checked(settings.flags & bs_frame);
	light_chooser.Set_Color(settings.colors[0]); 
	dark_chooser.Set_Color(settings.colors[1]); 
	last_move_chooser.Set_Color(settings.colors[2]); 
	frame_chooser.Set_Color(settings.colors[3]);
	return true;
}

void Board_Settings_Editor::Wdg_Close()
{
	Retrieve_Value();

	Object_Editor <Board_Settings> ::Wdg_Close();
}

void Board_Settings_Editor::Retrieve_Value()
{
	settings.flags= 0;
	if (show_names_check.Get_Checked())  settings.flags |= bs_show_names;
	if (turned_check.Get_Checked())      settings.flags |= bs_turned;
	if (anti_alias_check.Get_Checked())  settings.flags |= bs_anti_alias;
	if (frame_check.Get_Checked())       settings.flags |= bs_frame;

	settings.colors[0]= light_chooser.Get_Color();
	settings.colors[1]= dark_chooser.Get_Color();
	settings.colors[2]= last_move_chooser.Get_Color(); 
	settings.colors[3]= frame_chooser.Get_Color(); 
}
