#ifndef BUFFER_INCLUDED
#define BUFFER_INCLUDED

#if 0
/* A buffer is a top-level widget.  If provides a few standard
   messages, which are defined in Popup_Window in this
   implementation. 
 */
#endif /* 0 */ 

#include "dynamic.hh"
#include "vector.hh"
#include "widget.hh"

class Buffer
{
public:
  
	template <class Type>
	static bool
	Open_Buffer(typename Type::Open_Info open_info);

	static void Flush_Closed_Buffers();
	/* Call one time after all windows have been closed. 
	 */ 

	static int Get_Buffer_Count() 
		{
			return buffers.Get_Size();
		}
	static int Get_Last_Opened()
		{
			Assert (last_opened >= 0);
			return last_opened; 
		}
	static Widget &Get(int i)
		{
			return buffers[i](); 
		}
private:

	static Vector <Dynamic <Widget> > buffers;
	/* Buffers that been opened with Open_Buffer().  Closed and
	   destroyed by Flush_Closed_Buffers().  Entries may be NULL. 
	*/
	static int last_opened; 
};

template <class Type>
bool
Buffer::Open_Buffer(typename Type::Open_Info open_info)
{
	Vector <Dynamic <Widget> > &buf= buffers; 

	(void)buf; 
	
	int size= *buffers; 
	if (! buffers.Set(size + 1))
		return false;
	Dynamic <Type> new_buffer;
	if (! new_buffer.Alloc())
	{
		buffers= -1; 
		return false;
	}
	/* Open() may open buffers, therefore BUFFERS may have changed
	   size. 
	*/
	if (! new_buffer().Open(NULL, open_info))
	{
		buffers[size].Free();
		return false;
	}

	new_buffer.Transfer_To(buffers[size]);
	last_opened= size; 
	return true; 
}


#endif /* ! BUFFER_INCLUDED */
