#include "check_position.hh"

#include "trl.hh"

int Check_Position(const Position &p)
{
	/* castlings */
//	p.moved[1]= 0;
//	p.moved[4]= 0;

	/* number of kings */
	int king_count[2]= {0, 0};
	for (int file= sq_a;  file <= sq_h; ++ file)
		for (int rank= sq_1; rank <= sq_8; ++ rank)
		{
			Piece e= p.Get_Piece(Make_Square(file, rank));
			if (e.Is_Type(pt_K))
			{
				++ king_count[e.Get_Color()];
//				p.Get_King_Square(e.Get_Color())= Make_Square(file, rank);
			}
		}
	if (king_count[0] == 0)
		return nrm_no_white_king;
	if (king_count[1] == 0)
		return nrm_no_black_king;
	if (king_count[0] > 1)
		return nrm_multiple_white_kings;
	if (king_count[1] > 1)
		return nrm_multiple_black_kings;

	Assert (p[p.Get_King_Square(cc_w)] == Piece(cc_w, pt_K));
	Assert (p[p.Get_King_Square(cc_b)] == Piece(cc_b, pt_K));

	/* King can be captured. 
	 */
	if (p.Square_Attacked(p.Get_King_Square(CC_Turn(p.Get_To_Move())),
			      CC_Turn(p.Get_To_Move())))
		return nrm_king_in_capture;

	/* pawn on first row
	 */
	for (int file= sq_a;  file <= sq_h;  ++ file)
		if (p.Get_Piece(Make_Square(file, sq_1)).Is_Piece_Of_Type(pt_P)
		    || p.Get_Piece(Make_Square(file, sq_8)).Is_Piece_Of_Type(pt_P))
			return nrm_pawn_on_first_row;

//	p.ep_square= 0;

	return nrm_ok;
}

String
Check_Position_Error(int e)
{
	Assert (e >= nrm_no_white_king && e <= nrm_pawn_on_first_row);

	static const char *const _messages[]= {"OK",
					       "No white king",
					       "No black king",
					       "Multiple white kings",
					       "Multiple black kings",
					       "King in capture",
					       "Pawn on first or last row"};
	return Trl(_messages[e]);
}
