#include "client_image.hh"

#include "dbg.hh"

bool
Client_Image::Send_Rect(Canvas &canvas, 
			Rect dst,
			Point src)
{
	Assert (Is());

	gdk_draw_image(canvas.Get_Drawable(),
		       canvas.Get_GC(),
		       image,
		       src.x, src.y,
		       dst.origin.x, dst.origin.y,
		       dst.size.x, dst.size.y);

	return true; 
}

bool
Client_Image::Receive_Rect(const Canvas &canvas,
			   Rect rect)
{
	Assert (! Is());

	image= gdk_image_get(canvas.Get_Drawable(),
			     rect.origin.x,
			     rect.origin.y,
			     rect.size.x,
			     rect.size.y);

	return image != NULL; 
}

bool
Client_Image::
Draw_Stretch_Canvas(Canvas &dst,
		    const Canvas &src,
		    Rect &dst_rect,
		    Rect &src_rect,
		    bool anti_alias)
{
	Dbg_Function();
	Dbg_Int(anti_alias);

	Client_Image src_image;

	if (! src_image.Receive_Rect(src, src_rect))
		return false;

	Client_Image dst_image;

	if (! dst_image.Create(dst_rect.size))
		return false;

  
	for (int x= 0; x < dst_rect.size.x; ++ x) 
	{
		for (int y= 0; y < dst_rect.size.y; ++ y)
		{
			int r= 0, g= 0, b= 0;
			if (anti_alias)
			{
				for (int xx= src_rect.size.x * x / dst_rect.size.x;
				     xx < src_rect.size.x * (x + 1) / dst_rect.size.x;
				     ++ xx)
					for (int yy= src_rect.size.y * y / dst_rect.size.y;
					     yy < src_rect.size.y * (y + 1) / dst_rect.size.y;
					     ++ yy)
					{
						RGB rgb;
						src_image.Get_Pixel(xx, yy,
								    rgb.r, rgb.g, rgb.b);
						r += rgb.r;
						g += rgb.g;
						b += rgb.b;
					}
				int count= ((src_rect.size.x * (x + 1) / dst_rect.size.x
					     - src_rect.size.x * x / dst_rect.size.x)
					    * (src_rect.size.y * (y + 1) / dst_rect.size.y
					       - src_rect.size.y * y / dst_rect.size.y));
				r /= count;
				g /= count;
				b /= count;
			}
			else
			{
				src_image.Get_Pixel(src_rect.size.x * x / dst_rect.size.x,
						    src_rect.size.y * y / dst_rect.size.y,
						    r, g, b);
			}

			dst_image.Draw_Pixel(x, y,
					     r, g, b); 
		}
	}


	if (! dst_image.Send_Rect(dst, dst_rect, Point(0)))
		return false;

	return true; 
}

