#ifndef DBG_INCLUDED
#define DBG_INCLUDED

/* Macros that output debugging information.  Only enabled when
   <MODULE>_DBG is defined.  
 */

/* Dbg(format, [args...])

   Print message.
 */
#ifndef NDEBUG

#include "debug.hh"
#include "case.hh"
#include "env.hh"

#define Dbg(args...) 					        \
((void)(({							\
	String _dbg_name= __FILE__;					\
	_dbg_name= _dbg_name.Sub(4, *_dbg_name - 7);				\
	_dbg_name= Case_Up(_dbg_name);					\
	_dbg_name += "_DBG";						\
	String _dbg_value= Env::Get(_dbg_name);				\
	if (_dbg_value.Is())						\
	{							\
                Debug(args);\
	}							\
        0; \
}) == 0))

#else /* NDEBUG */
#    define Dbg(args...)  ((void) 0)
#endif

/* Dbg_Function()

   Print function name.
 */
#define Dbg_Function()     \
  Dbg("%s", __PRETTY_FUNCTION__)


/* Print an integer.
 */
#define Dbg_Int(arg) Dbg("\t" #arg "=  %d", arg)


/* Print string.
 */
#define Dbg_Str(arg)  Dbg("\t" #arg "=  \"%s\"", arg)

#define Dbg_Pointer(arg) Dbg("\t" #arg "=  %p", arg)

/* Macros that end in an underscore:  do nothing. 
 */
#define Dbg_(args...) 
#define Dbg_Function_()
#define Dbg_Int_(arg)
#define Dbg_Str_(arg)
#define Dbg_Pointer_(arg)

#endif /* ! DBG_INCLUDED */
