#include "game_archive.hh"

#include "pgn.hh"

bool
Game_Archive::Set_Size(int n) 
{
	int size= *games;
	if (! games.Set(n))
		return false;
	if (! raw.Set(n))
	{
		games.Set(size);
		return false;
	}
	return true;
}

bool
Game_Archive::Check(int n)
{
	if (games[n].Is())
		return true;

	if (! games[n].Alloc())
		return false;

	if (! PGN::Read_Raw(raw[n], games[n]()))
	{
		games[n].Free();
		return false;
	}

	return true;
}

bool
Game_Archive::Insert_Game(int n)
{
	Assert (n >= 0 && n <= *raw);
	Assert (*raw == *games); 

	int size= *raw;
	if (! raw.Set(size + 1))
		return false;
	if (! games.Set(size + 1))
	{
		raw.Set(size);
		return false;
	}

	for (int i= n;  i < size;  ++i)
	{
		raw[i + 1]= raw[i];
		games[i].Transfer_To(games[i + 1]);
	}
	/* Let RAW be open on the new game. 
	 */
	raw[n].Reset();
	raw[n].Normalize();

	Assert (! games[n].Is());

	return true;
}

void
Game_Archive::Remove(int n)
{
	int size= *raw;

	Assert (n >= 0 && n < size);

	for (int i= n;  i < size - 1;  ++i)
	{
		raw[i]= raw[i + 1];
		games[i + 1].Transfer_To(games[i]);
	}

	raw.Set(size - 1);
	games.Set(size - 1);
}

void
Game_Archive::Set(int n,
		  const Game &game)
{
	Assert (n >= 0 && n < *raw);

	games[n].Free();
	if (! games[n].Alloc(game))
	{
		raw[n].Reset();
	}
}

String Game_Archive::Get_Result(int i)
{
	Dbg_Function();
	Dbg_Int(i);
	Dbg_Int(games[i].Is());

	Assert (i >= 0 && i < *raw);

	if (games[i].Is())
		return games[i]().Get_Game_Info()
			.Get_Result_Text();
	else
		return raw[i].Get_Tag("Result"); 
}
