#include "generate_legal.hh"

#include "deref.hh"
#include "moves.hh"
#include "dbg.hh"

static bool gen_leg_just_check;
//static int gen_leg_state;
/* 1:    Generate all moves. 
   2:    Just check whether there are moves. 
   {   3:    Must terminate. }
*/

static bool gen_leg_strict_legal;
/* Whether checking for only legal moves. 
 */

static Pointer <Position> gen_leg_position;
static Pointer <Vector <Move> > gen_leg_moves;

static int gen_leg_square;
/* Current square. 
 */

static Chess_Color gen_leg_to_move;
/* Side to move
 */

static int gen_leg_ep_poss;
/* Of analized position. 
 */

int gen_flags;

bool
Gen_Leg_Add_Move(int to, 
		 Piece captured_piece,
		 Piece promotion, 
		 int ep_capture, 
		 Move::Castling castling= Move::no_castling)
	/* Return value:  whether generation should terminate. 
	 */
{
	if (gen_leg_moves().Set(*gen_leg_moves() + 1))
	{
		Pointer <Move> move= gen_leg_moves()[*gen_leg_moves() - 1];
		move().from= gen_leg_square;
		move().to= to;
		move().captured_piece= captured_piece;
		move().ep_poss_square= gen_leg_ep_poss;
		move().promotion= promotion;
		move().ep_square= ep_capture;
		move().castling= castling;
		move().piece= gen_leg_position().Get_Piece(gen_leg_square);
		move().halfmove_clock= gen_leg_position().Get_Halfmove_Clock();

		bool append= true;
		/* Wether the move must be appended.  Set to FALSE
		   when it is not legal and only legal moves are
		   generated.  
		*/
	
		if (gen_leg_strict_legal)
		{
			gen_leg_position() << move();
			Chess_Color cc= CC_Turn(gen_leg_position().Get_To_Move());

			append= ! gen_leg_position().Square_Attacked
				(gen_leg_position().Get_King_Square(cc), 
				 cc);

			gen_leg_position() >> move();
		}
	
		if (append)
		{
			if (gen_leg_just_check)
				return true;
		}
		else
		{
			move= NULL;
			gen_leg_moves().Set(*gen_leg_moves() - 1);
			return false;
		}
	}
	return false;
}

bool
Gen_Leg_Pawn(Piece from_piece)
{
	if (from_piece.Get_Color() == cc_w)
		/* white pawn */
	{
		if (Square_Rank(gen_leg_square) == sq_7)
			/* white pawn on 7th rank */
		{
			int osize= *gen_leg_moves();
			if (! gen_leg_position().Get_Piece(gen_leg_square + 10).Is())
			{
				for (Piece_Type pt= pt_Q;
				     pt >= pt_N;  
				     --pt)
					if (Gen_Leg_Add_Move
						(gen_leg_square + 10,
						 Piece(false),
						 Piece(cc_w, pt), 0))
						return true;
			}
			if (gen_leg_position().Get_Piece(gen_leg_square + 9)
			    .Is_Piece_Of_Color(cc_b))
			{
				for (Piece_Type g= pt_Q;
				     g >= pt_N;
				     -- g)
					if (Gen_Leg_Add_Move
						(gen_leg_square + 9,
						 gen_leg_position().Get_Piece
						 (gen_leg_square + 9),
						 Piece(cc_w, g), 0))
						return true;
			}
			if (gen_leg_position().Get_Piece(gen_leg_square + 11) 
			    .Is_Piece_Of_Color(cc_b))
			{
				for (Piece_Type g= pt_Q;
				     g >= pt_N;
				     -- g)
					if (Gen_Leg_Add_Move
						(gen_leg_square +11,
						 gen_leg_position().Get_Piece
						 (gen_leg_square + 11),
						 Piece(cc_w, g), 0))
						return true;
			}
			if (osize != *gen_leg_moves())
				gen_flags |= gn_promotion;
		}
		else
		{
			if (! gen_leg_position().Get_Piece(gen_leg_square + 10).Is())
				if (Gen_Leg_Add_Move
					(gen_leg_square + 10,
					 Piece(false), 0, 0))
					return true;
			if (Square_Rank(gen_leg_square) == sq_2   &&
			    gen_leg_position().Get_Piece(gen_leg_square + 10) == Piece(false) &&
			    gen_leg_position().Get_Piece(gen_leg_square + 20) == Piece(false))
				if (Gen_Leg_Add_Move
					(gen_leg_square + 20,
					 Piece(false), 0, 0))
					return true;
			if (gen_leg_position().Get_Piece(gen_leg_square + 9)
			    .Is_Piece_Of_Color(cc_b))
				if (Gen_Leg_Add_Move
					(gen_leg_square + 9,
					 gen_leg_position().Get_Piece(gen_leg_square + 9), 0, 0))
					return true;
			if (gen_leg_position().Get_Piece(gen_leg_square + 11)
			    .Is_Piece_Of_Color(cc_b))
				if (Gen_Leg_Add_Move
					(gen_leg_square + 11,
					 gen_leg_position().Get_Piece(gen_leg_square+11), 0, 0))
					return true;
			if (Square_Rank(gen_leg_square) == sq_5)
			{
				if (gen_leg_position().Get_EP_Square() == gen_leg_square - 1)
					if (Gen_Leg_Add_Move
						(gen_leg_square + 9,
						 Piece(cc_b, pt_P), 0,
						 gen_leg_square - 1))
						return true;
				if (gen_leg_position().Get_EP_Square() == gen_leg_square + 1)
					if (Gen_Leg_Add_Move
						(gen_leg_square + 11,
						 Piece(cc_b, pt_P), 0,
						 gen_leg_square + 1))
						return true;
			}
		}
	}
	else /* black pawn */
	{
		if (Square_Rank(gen_leg_square) == sq_2)
			/* black pawn on second rank */
		{
			int osize= *gen_leg_moves();
			if (! gen_leg_position().Get_Piece(gen_leg_square - 10).Is())
			{
				for (Piece_Type g= pt_Q;
				     g >= pt_N;
				     -- g)
					if (Gen_Leg_Add_Move
						(gen_leg_square - 10,
						 Piece(false),
						 Piece(cc_b, g), 0))
						return true;
			}
			if (gen_leg_position().Get_Piece(gen_leg_square - 9)
			    .Is_Piece_Of_Color(cc_w))
			{
				for (Piece_Type g= pt_Q;
				     g >= pt_N;
				     -- g)
					if (Gen_Leg_Add_Move
						(gen_leg_square - 9,
						 gen_leg_position().Get_Piece(gen_leg_square - 9),
						 Piece(cc_b, g), 0))
						return true;
			}
			if (gen_leg_position().Get_Piece(gen_leg_square - 11)
			    .Is_Piece_Of_Color(cc_w))
			{
				for (Piece_Type g= pt_Q;
				     g >= pt_N;
				     -- g)
					if (Gen_Leg_Add_Move
						(gen_leg_square - 11,
						 gen_leg_position().Get_Piece(gen_leg_square - 11),
						 Piece(cc_b, g), 0))
						return true;
			}
			if (osize != *gen_leg_moves())
				gen_flags |= gn_promotion;
		}
		else /* other black pawn */
		{
			if (! gen_leg_position().Get_Piece(gen_leg_square - 10).Is())
				if (Gen_Leg_Add_Move
					(gen_leg_square - 10,
					 Piece(false), 0, 0))
					return true;
			if (Square_Rank(gen_leg_square) == sq_7
			    && (gen_leg_position().Get_Piece(gen_leg_square - 10)
				== Piece(false))
			    && (gen_leg_position().Get_Piece(gen_leg_square - 20)
				== Piece(false)))
				if (Gen_Leg_Add_Move
					(gen_leg_square - 20,
					 Piece(false), 0, 0))
					return true;
			if (gen_leg_position().Get_Piece(gen_leg_square - 11)
			     .Is_Piece_Of_Color(cc_w))
				if (Gen_Leg_Add_Move
					(gen_leg_square-11,
					 gen_leg_position().Get_Piece(gen_leg_square - 11), 0, 0))
					return true;
			if (gen_leg_position().Get_Piece(gen_leg_square - 9)
			    .Is_Piece_Of_Color(cc_w))
				if (Gen_Leg_Add_Move
					(gen_leg_square - 9,
					 gen_leg_position().Get_Piece(gen_leg_square - 9), 0, 0))
					return true;
			if (Square_Rank(gen_leg_square) == sq_4)
			{
				if (gen_leg_position().Get_EP_Square() == gen_leg_square - 1)
					if (Gen_Leg_Add_Move
						(gen_leg_square - 11,
						 Piece(cc_w, pt_P), 0,
						 gen_leg_square - 1))
						return true;
				if (gen_leg_position().Get_EP_Square() == gen_leg_square + 1)
					if (Gen_Leg_Add_Move
						(gen_leg_square - 9,
						 Piece(cc_w, pt_P), 0,
						 gen_leg_square + 1))
						return true;
			}
		}
	}
	
	return false;
}

void
Generate_Legal(Position &position,
	       Vector <Move> &moves,
	       bool strict_legal, 
	       bool just_check)
{
	Dbg_Function();

	gen_leg_moves= moves;

	gen_leg_just_check= just_check;
	gen_leg_strict_legal= strict_legal;
	gen_leg_moves= moves;
	gen_leg_position= position;

	int i;
	int j0, j1;

	gen_leg_to_move= position.Get_To_Move();
	gen_leg_ep_poss= position.Get_EP_Square();
	gen_flags= 0;
  
	for (int rank= sq_1;  
	     rank <= sq_8;
	     ++ rank)
	{
		for (int file= sq_a;
		     file <= sq_h;
		     ++ file)
		{
			gen_leg_square= Make_Square(file, rank);
			Piece from_piece= position.Get_Piece(gen_leg_square);
			if (from_piece.Is_Piece_Of_Color(gen_leg_to_move))
			{
				switch (from_piece.Get_Type())
				{
				case pt_P: /* pawn */
					if (Gen_Leg_Pawn(from_piece))
						goto end;
					break;
				case pt_N: /* knight */
					for (i= 8; i<16 /*&& gen_leg_state!=3*/; ++i)
					{
						int f= gen_leg_square + moves_offset[i]; 
						Piece g= position.Get_Piece(f);
						if (position.Get_Piece(f).Can_Color_Move_To
						    (gen_leg_to_move))
							if (Gen_Leg_Add_Move(f, g, 0, 0))
								goto end;
					}
					break;

				case pt_B:  /* long move */
				case pt_R:     
				case pt_Q:
				{
					Piece_Type fig0= position.Get_Piece
						(gen_leg_square).Get_Type(); 
					j0= moves_range[0][fig0 - 2];
					j1= moves_range[1]
						[position.Get_Piece(gen_leg_square).Get_Type() - 2]; 

					for (i= j0; i<j1 /* && gen_leg_state != 3*/; ++i) 
					{
						int f= gen_leg_square + moves_offset[i];
						while (position.Get_Piece(f).Is_Empty()
						       /* && (gen_leg_state != 3)*/)
						{
							if (Gen_Leg_Add_Move
							    (f, Piece(false), 0, 0))
								goto end;
							f += moves_offset[i];
						}
						Piece g= position.Get_Piece(f);
						if (g.Is_Piece_Of_Color
						    (CC_Turn(gen_leg_to_move))
						    /* && gen_leg_state != 3 */ )
							if (Gen_Leg_Add_Move(f, g, 0 , 0))
								goto end;
					}
					break;
				}

				case pt_K: /* king */
		  
					for (i= 0; i < 8; ++i)
					{
						int f= gen_leg_square + moves_offset[i];
						Piece g= position.Get_Piece(f);
						if (g.Can_Color_Move_To(gen_leg_to_move))
							if (Gen_Leg_Add_Move(f, g, 0, 0))
								goto end;
					}
					if (gen_leg_square == sq_e1 && !position.Get_Moved(1)
					    && !position.Square_Attacked(sq_e1, cc_w))
						/* white castling */
					{
						if (position.Get_Piece(sq_f1) == Piece(false)
						    && position.Get_Piece(sq_g1) == Piece(false)
						    && !position.Square_Attacked(sq_f1, cc_w)
						    && !position.Get_Moved(2))
							if (Gen_Leg_Add_Move(sq_g1, Piece(false), 0, 0,  
									     Move::w_O_O))
								goto end;
						if (position.Get_Piece(sq_d1) == Piece(false)
						    && position.Get_Piece(sq_c1) == Piece(false)
						    && position.Get_Piece(sq_b1) == Piece(false)
						    && !position.Square_Attacked(sq_d1, cc_w)
						    && !position.Get_Moved(0))
							if (Gen_Leg_Add_Move(sq_c1, Piece(false), 0, 0, 
									     Move::w_O_O_O))
								goto end;
					}
					else
						if (gen_leg_square == sq_e8 && !position.Get_Moved(4)
						    && !position.Square_Attacked(sq_e8, cc_b))
							/* black castling */
						{
							if (position.Get_Piece(sq_f8) == Piece(false) &&
							    position.Get_Piece(sq_g8) == Piece(false) &&
							    ! position.Square_Attacked(sq_f8, cc_b) &&
							    ! position.Get_Moved(5))
								if (Gen_Leg_Add_Move
								    (sq_g8, Piece(false),
								     0, 0, Move::b_O_O))
									goto end;

							if (position.Get_Piece(sq_d8) == Piece(false)
							    && position.Get_Piece(sq_c8) == Piece(false)
							    && position.Get_Piece(sq_b8) == Piece(false)
							    && !position.Square_Attacked(sq_d8, cc_b)
							    && !position.Get_Moved(3))
								if (Gen_Leg_Add_Move
								    (sq_c8, Piece(false), 0, 0,
								     Move::b_O_O_O))
									goto end;
						}
					break;

				default:
					Assert_Unreached();
				}
			}
		}
	}

 end:

	gen_leg_position= NULL;
	gen_leg_moves= NULL;
}
