#include "gtk_object.hh"

#include "dbg.hh"
#include "vector.hh"
#include "input_reader.hh"
#include "gtk_timer.hh"

gint
Gtk_Object::Signal_Delete(GtkWidget *this_gtkwidget,
			  GdkEvent  * /* event */ ,
			  gpointer   data)
{
	/* Convert DATA to GTK_OBJECT and check
	   consistency.  
	*/

	Gtk_Object *gtk_object= (Gtk_Object *)data;
	Assert (gtk_object != NULL);
	Assert (Deref(gtk_object).gtkwidget == this_gtkwidget);

	if ((! Deref(gtk_object).force_close) && 
	    (! Deref(gtk_object).Wdg_Close_Query()))
		return TRUE;
	Deref(gtk_object).deleted= true;

	return FALSE;
}

void
Gtk_Object::Signal_Destroy(GtkWidget *,
			   gpointer data)
{
	Gtk_Object &gtk_object= *((Gtk_Object *)data); 
	gtk_object.Wdg_Gtk_Delete(); 
	gtk_object.Wdg_Close(); 
	gtk_object.gtkwidget= NULL;
	gtk_object.Wdg_Destroy();
}

void
Gtk_Object::Signal_Expose(GtkWidget *this_gtkwidget,
			  GdkEventExpose * /* event */ ,
			  gpointer data)
{
	Gtk_Object *gtk_object= (Gtk_Object *)data;
	Assert (gtk_object != NULL);
	Assert (Deref(gtk_object).gtkwidget == this_gtkwidget);

	Canvas canvas(this_gtkwidget->window,
		      this_gtkwidget->style->fg_gc[GTK_WIDGET_STATE(this_gtkwidget)]); 
	Deref(gtk_object).Wdg_Draw(canvas); 
	canvas.Set_Draw_Mode(Canvas::dm_copy);
	canvas.Set_Color(Canvas::Get_Black());
}

gint
Gtk_Object::Signal_Motion_Notify_Event(GtkWidget *this_gtkwidget,
				       GdkEventMotion *event,
				       gpointer data)
{
	Dbg_Function_();

	Assert (event != NULL);

	Gtk_Object *gtk_object= (Gtk_Object *)data;
	Assert (gtk_object != NULL);
	Assert (Deref(gtk_object).gtkwidget == this_gtkwidget);

	
	int x, y;
	GdkModifierType state;

	if (Deref(event).is_hint)
		gdk_window_get_pointer (event->window, &x, &y, &state);
	else
	{
	        x = /*lrint*/(int)(event->x);
		y = /*lrint*/(int)(event->y);
//		state = event->state;
	}

	Deref(gtk_object).Wdg_Mouse(ma_move,
				    mb_left,
				    Point(x, y),
				    true);
	Deref(gtk_object).Wdg_Mouse_Cursor();

	return TRUE; 
}

gint
Gtk_Object::Signal_Button_Press_Event(GtkWidget *this_gtkwidget,
				      GdkEventButton *event,
				      gpointer data)
{
	Dbg_Function_();

	Gtk_Object *gtk_object= (Gtk_Object *)data;
	Assert (gtk_object != NULL);
	Assert (Deref(gtk_object).gtkwidget == this_gtkwidget);
	
	Mouse_Button buttons[4]= {mb_left, mb_left, mb_middle, mb_right};

	if (Deref(event).button < 1  ||
	    Deref(event).button > 3)
		return TRUE;
	Deref(gtk_object).Wdg_Mouse(ma_down, 
				    buttons[Deref(event).button],
				    Point(/*lrint*/(int)(Deref(event).x),
					  /*lrint*/(int)(Deref(event).y)),
				    true);
	return TRUE;
}

gint
Gtk_Object::Signal_Button_Release_Event(GtkWidget *this_gtkwidget,
					GdkEventButton *event,
					gpointer data)
{
	Dbg_Function_();

	Gtk_Object *gtk_object= (Gtk_Object *)data;
	Assert (gtk_object != NULL);
	Assert (Deref(gtk_object).gtkwidget == this_gtkwidget);
	
	Mouse_Button buttons[4]= {mb_left, mb_left, mb_middle, mb_right};

	if (Deref(event).button < 1  ||
	    Deref(event).button > 3)
		return TRUE;
	Deref(gtk_object).Wdg_Mouse(ma_up, 
				    buttons[Deref(event).button],
				    Point((int)/*lrint*/(Deref(event).x),
					  (int)/*lrint*/(Deref(event).y)),
				    true);
	return TRUE; 
}

void
Gtk_Object::Attach(GtkWidget *new_gtkwidget)
{
	Assert (new_gtkwidget != NULL);
	Assert (gtkwidget == NULL);
	gtkwidget= new_gtkwidget;

	gtk_signal_connect(GTK_OBJECT(gtkwidget), "delete_event",
			   GTK_SIGNAL_FUNC(Signal_Delete), this);
	gtk_signal_connect(GTK_OBJECT(gtkwidget), "destroy",
			   GTK_SIGNAL_FUNC(Signal_Destroy), this);
	gtk_signal_connect(GTK_OBJECT(gtkwidget), "expose_event",
			   GTK_SIGNAL_FUNC(Signal_Expose), this); 
	gtk_signal_connect(GTK_OBJECT(gtkwidget), "size-allocate",
			   GTK_SIGNAL_FUNC(Signal_Size_Allocate), this); 
	gtk_signal_connect(GTK_OBJECT(gtkwidget), "size-request",
			   GTK_SIGNAL_FUNC(Signal_Size_Request), this); 


	force_close= false;
	/* For widgets for which Signal_Destroy() is
	   called automatically. 
	*/
	
	deleted= false;
}

void 
Gtk_Object::Close(bool force)
{
	Assert (gtkwidget != NULL);

	force_close= force; 

	Wdg_Do_Close();
}

void
Gtk_Object::Signal_Size_Allocate(GtkWidget *this_gtkwidget,
				 GtkAllocation *allocation,
				 gpointer data)
{
	Gtk_Object *gtk_object= (Gtk_Object *)data;
	Assert (gtk_object != NULL);
	Assert (Deref(gtk_object).gtkwidget == this_gtkwidget);

	Deref(gtk_object).Wdg_Size(Point(Deref(allocation).width,
					 Deref(allocation).height)); 
}

void
Gtk_Object::Signal_Size_Request(GtkWidget *this_gtkwidget,
				GtkRequisition *requisition,
				gpointer data)
{
	Dbg_Function_();
	Gtk_Object *gtk_object= (Gtk_Object *)data;
	Assert (gtk_object != NULL);
	Assert (Deref(gtk_object).gtkwidget == this_gtkwidget);

	int w= Deref(requisition).width;
	int h= Deref(requisition).height;
	bool resizeable= false;
	Deref(gtk_object).Wdg_Get_Size_Request(w, h, resizeable);
	Deref(requisition).width= w;
	Deref(requisition).height= h; 
}

class Gtk_Input_Info
{
public:

	Pointer <Stream> stream;
	guint id;
};

static Vector <Gtk_Input_Info> gtk_input_vector;

static
void Gtk_Object_Input_Function(gpointer data,
			       gint /* source */ ,
			       GdkInputCondition gic)
{
  Dbg_Function();

	if (! (gic & GDK_INPUT_READ))
		return; 

	Input_Reader &ir= Deref((Input_Reader *)data);
  
	ir.IR_Input();
}

int Register_Input(Input_Reader &ir)
{
	guint id= gtk_input_add_full
	  (ir.Get_Stream().Get_FD(),
	   GDK_INPUT_READ,
	   & Gtk_Object_Input_Function,
	   NULL,
	   & ir,
	   NULL);

	return id; 
}

void Unregister_Input(Input_Reader &ir)
{
	Dbg_Function();
	gtk_input_remove(ir.Get_ID()); 
}

void Gtk_Object::Connect_Key_Signals(GtkWidget *widget)
{
  Dbg_Function();
  gtk_widget_add_events(widget, GDK_KEY_PRESS); 

  gtk_signal_connect(GTK_OBJECT(gtkwidget), "key-press-event",
		     GTK_SIGNAL_FUNC(Signal_Key_Press_Event), this);
}

void Gtk_Object::Connect_Mouse_Signals(GtkWidget *gtkwidget)
{
	Dbg_Function_();

	gtk_widget_add_events(gtkwidget,
//			      GDK_EXPOSURE_MASK |
			      GDK_POINTER_MOTION_MASK |
			      GDK_POINTER_MOTION_HINT_MASK |
			      GDK_BUTTON_MOTION_MASK  |
			      GDK_BUTTON_PRESS_MASK   |
			      GDK_BUTTON_RELEASE_MASK);

	gtk_signal_connect(GTK_OBJECT(gtkwidget), "motion-notify-event",
			   GTK_SIGNAL_FUNC(Signal_Motion_Notify_Event), this); 
	gtk_signal_connect(GTK_OBJECT(gtkwidget), "button-press-event",
			   GTK_SIGNAL_FUNC(Signal_Button_Press_Event), this); 
	gtk_signal_connect(GTK_OBJECT(gtkwidget), "button-release-event",
			   GTK_SIGNAL_FUNC(Signal_Button_Release_Event), this); 
}

gint Gtk_Object::Signal_Key_Press_Event(GtkWidget *,
					   GdkEventKey *e,
					   gpointer data)
{
  Dbg_Function();

  Gtk_Object &o= Deref((Gtk_Object *)data);

  o.Wdg_Key_Press(Key(e));
  return TRUE; 
}

