#ifndef GUI_MAIN_INCLUDED
#define GUI_MAIN_INCLUDED

/* Program that opens one window and terminates when all windows are
   closed. 
 */

#include "buffer.hh"
#include "arg_main.hh"
#include "io.hh"
#include "error.hh"
#include "main.hh"
#include "format.hh"
#include "inform.hh"
#include "widget.hh"
#include "write_formatted.hh"

template <class Type>
int
Gui_Main(int /* argc */ , 
	 char **argv)
{
	int result= 0;

	bool success= Buffer::Open_Buffer <Type> 
		(typename Type::Open_Info(argv[1])); 


	if (! success)
	{
		String name= Main_Get_Name();
		String error= Error::Get();
		String message;
		if (argv[1] == NULL)
		{
			message= error;
		}
		else
		{
			message= Format("cannot open `%s':  %s",
					argv[1],
					error());
		}

		Write_Formatted(IO::err,
				"%s:  %s.\n",
				name(),
				message());
		result= 1;
	}

	Widget::Loop();					
	Buffer::Flush_Closed_Buffers();	
	Assert (Buffer::Get_Buffer_Count() == 0);

	return result;
}

#define GUI_MAIN(Type)					\
							\
const char *						\
arg_get_help=\
        Type::Get_Help();                      \
							\
int							\
Arg_Main(int argc,					\
	 char **argv)					\
{							\
        return Gui_Main <Type> (argc, argv);            \
}							\

//#endif /* GUI_MAIN_DEFAULT */

#endif /* ! GUI_MAIN_INCLUDED */
