#include "ics_window.hh"

#include "keys.hh"
#include "inform.hh"
#include "format.hh"
#include "error.hh"
#include "keys.hh"
#include "rc.hh"
#include "generator.hh"
#include "parse_move.hh"
#include "about.hh"
#include "ics_account_stream.hh"
#include "notation.hh"

bool
ICS_Window::Open(Pointer <Widget> parent, 
		 Open_Info)
{
	if (! V_Packer::Open(parent))
		return false;

	Menu_Window::Open_Menu(this);

	This.Set_Next_Resizeable(true);
	if (! top_packer.Open(this))
	  goto close;

	if (! side_packer.Open(top_packer))
	  goto close;

	if (! black_packer.Open(side_packer))
	  goto close;

	black_packer.Set_Next_Resizeable(true);
	if (! label_black.Open(black_packer,
			       Trl("Black"),
			       dt_text,
			       st_background,
			       al_begin))
	  goto close;
	if (! black_clock_packer.Open(black_packer))
	  goto close;
	if (! black_clock.Open(black_clock_packer))
	  goto close;
	if (! black_text_clock.Open(black_clock_packer,
				    "",
				    dt_code,
				    st_background,
				    al_center))
	  goto close;

	if (! white_packer.Open(side_packer))
	  goto close;

	white_packer.Set_Next_Resizeable(true);
	if (! label_white.Open( white_packer,
			       Trl("White"),
			       dt_text,
			       st_background,
			       al_begin))
	  goto close; 
	if (! white_clock_packer.Open(white_packer))
	  goto close;
	if (! white_clock.Open(white_clock_packer))
	  goto close;
	if (! white_text_clock.Open(white_clock_packer, 
				    "",
				    dt_code,
				    st_background,
				    al_center))
	  goto close; 

//	right_packer.Set_Next_Resizeable(true);
//	legal_move_list.Open(right_packer, msg_legal_move_list);

	material_widget.Open(side_packer);

	top_packer.Set_Next_Resizeable(true);
	if (! board.Open(top_packer,
			   ICS::Get_Position(),
			   msg_board))
	  goto close;

	if (! status_bar.Open(this, 2))
	  goto close;

	if (RC_Load(ics_account))
	{
		Connect();
	}
	else
	{
		ics_account.Reset();
		account_editor.Open(msg_account_edited, 
				    This);
		if (status_bar.Is(0))
			status_bar.Set(0, Trl("Not connected")); 
	}

	persistent_notation.Open(This, msg_notation);

	return true;

      close:
	V_Packer::Close(); 
	return false;
}

void
ICS_Window::Wdg_Close()
{
	Logout();
	account_editor.Close();
	if (Timer::Is())
		Timer::Stop();
	if (interaction_window.Is())
	  interaction_window.Close(true); 
	persistent_notation.Close(); 
	V_Packer::Wdg_Close();
}

void
ICS_Window::Menued_Get(int menu, 
		       Menued::Entries &entries)
{
	switch (menu)
	{
	default:
		Assert_Unreached();
		break;

	case menu_main:
		entries.Append_Submenu(menu_file, Trl("File"));
		entries.Append_Submenu(menu_view, Trl("View"));
		entries.Append_Submenu(menu_game, Trl("Game"));
		entries.Append_Submenu(menu_help, Trl("Help"));
		break;

	case menu_file:
		entries.Append_Command(msg_login,
				    Trl("Login"));
		entries.Append_Command(msg_logout,
				    Trl("Logout"));
		entries.Append_Separator();
		entries.Append_Command(V_Packer::msg_ok,
				    Trl("Quit"),
				    key_quit);
		break;

	      case menu_view:
		entries.Append_Checked
		  (msg_interaction,
		   Trl("Messages"),
		   interaction_window.Is() && interaction_window.Get_Visible());
		entries.Append_Checked(msg_show_legal,
				       Trl("Legal moves"),
				       legal_move_list.Is());
		entries.Append_Separator();
		entries.Set_Menued(persistent_notation);
		persistent_notation.Menued_Get
			(Persistent <Notation, Notation_Editor> ::menu_main,
			 entries);
		entries.Set_Menued(board());
		board().Menued_Get(Board::menu_main, entries);
		
		break;
	case menu_game:
		entries.Append_Command(msg_seek,
				    Trl("Seek"));
		entries.Append_Command(msg_resign,
				    "Resign");
		break;

	case menu_help:
		entries.Append_Command(msg_about,
				    Trl("About"),
				    key_about);
		break;

	}
}

void
ICS_Window::MR_Receive(int id)
{
	Dbg_Function();

	switch (id)
	{
	default:
		V_Packer::MR_Receive(id);
		break;

	case msg_board:
	{
	  Dbg("\tmsg_board");

		int from, to;
		if (board().Get_Squares(from, to))
		{
			Dbg_Int(from);
			Dbg_Int(to);


			if (ICS::Is() && ICS::Get_Generator().Is())
			  {
			    int sm= ICS::Get_Generator().Get_Square_Move(from,
									 to);
			    
			    Dbg_Int(sm);
			    if (sm != -1)
			      {
				if (promotion_chooser.Is())
				  {
				    int s= promotion_chooser.Get_Selected();
				    if (s != -1)
						sm += s;
				}
				
				User_Play(sm, from, to);
			      }
		      }

			if (to != 0)
			  board().Invalidate_Square(to);
			board().Invalidate_Square(from);
		}
	}
	break;

	case msg_account_edited:
		ics_account= account_editor.Get_Value();
		RC_Save(ics_account);
		Connect();
		account_editor.Have_Read();
		break;

	case msg_login:
		account_editor.Open(msg_account_edited, 
				    This,
				    ics_account);
		break;

	case msg_logout:
		Logout();
		break;

	case msg_seek:
	  if (ICS::Is() && ! ICS::Get_Inside_Game())
			Seek(5, 5);
		break;

	case msg_resign:
	  if (ICS::Is() && ICS::Get_Inside_Game())
			ICS::Resign();
		break;

	case msg_text:
	  if (! ICS::Is())
	    break;

		if (! interaction_window.Is())
		  break;
		ICS::Put(interaction_window.Pull_Text());
//			ICS::Put(edit.Get_Text());
//			edit.Set_Text("");

		break;

	case msg_about:
		About();
		break;

	      case msg_notation:
		Update_Position(); 
		break;
	      case msg_input:
	if (ICS::Proceed())
	{
		Logout();
		if (ICS::Get_Error().Is())
			Inform("Error",
			       ICS::Get_Error());
	}
		break;

	      case msg_interaction:
		if (! interaction_window.Is())
		  {
		    Open_Interaction_Window();
		    break;
		  }
//		interaction_window.Set_Visible(interaction_window.Get_Visible());
		interaction_window.Close(); 
		break;

	      case msg_legal_move_list:
		{
		  if (! legal_move_list.Is())
		    break;
		  int move= legal_move_list.Get_Selected();
		  if (move == -1)
		    break;
		  User_Play(move); 
		break;
			}

	      case msg_ok:
		Widget::Close(); 
		break;

	      case msg_show_legal:
		if (legal_move_list.Is())
		  {
		    legal_move_list.Close();
		  }
		else
		  {
		    side_packer.Set_Next_Resizeable(true);
		    if (! legal_move_list.Open(side_packer, msg_legal_move_list))
		      break;
		    Update_Legal_Moves();
		  }
		break;
	}
}

void
ICS_Window::Seek(int m, int s)
{
  Assert (! ICS::Get_Inside_Game());
	ICS::Seek(m, s);
	if (status_bar.Is(0))
		status_bar.Set(0, Trl("Seeking..."));
}

void
ICS_Window::Connect()
{
	Dbg_Function();


	if (ICS::Is())
		ICS::Close();

	Dbg_Str(ics_account.host());
	Dbg_Int(ics_account.port);
	Dbg_Str(ics_account.user());
	Dbg_Str(ics_account.password());

	if (! ICS::Open(ics_account.host,
			ics_account.port,
			ics_account.user,
			ics_account.password))
	{
		String e= Error::Get();
		Inform("Error",
		       Format("Cannot connect to `%s': %s",
			      ics_account.host(),
			      e()));
		Get_Window().Set_Caption(Trl("Disconnected"));
		return; 
	      }

	String user= ICS::Get_User();
	String host= ICS::Get_Host();
	Get_Window().Set_Caption
	  (Format("%s@%s",
		  user(),
		  host()));

	if (Input_Reader::Is())
	  Input_Reader::Close();
	Input_Reader::Open(ICS::Get_Output());
	if (status_bar.Is(0))
	  status_bar.Set(0, Format(Trl("Connected to %s"), host()));
//	Open_Interaction_Window();
}

void 
ICS_Window::Logout()
{
	if (ICS::Is())
	{
		Input_Reader::Close();
		ICS::Close();
		if (status_bar.Is(0))
			status_bar.Set(0, Trl("Not connected")); 
	}
}

void
ICS_Window::ICS_Changed()
{
	board().Invalidate_Position();
	if (status_bar.Is(0))
		status_bar.Set
			(0,
			 Trl(ICS::Get_Position().Get_To_Move() == cc_w
			     ? "White to move"
			     : "Black to move"));
	if (label_white.Is())
		label_white.Set_Style(ICS::Get_Position().Get_To_Move() == cc_w
				      ? st_selected : st_background);
	if (label_black.Is())
		label_black.Set_Style(ICS::Get_Position().Get_To_Move() == cc_b
				      ? st_selected : st_background);
}

void
ICS_Window::ICS_New_Game()
{
	bool turned= board().Get_Turned();
	bool new_turned= ICS::Get_Playing_Side();
	if (turned != new_turned)
	{
		board().Set_Turned(new_turned);
		if (white_packer.Is() && black_packer.Is())
		{
			if (new_turned)
			{
				white_packer.Set_Index(0);
				black_packer.Set_Index(1);
			}
			else
			{
				black_packer.Set_Index(0);
				white_packer.Set_Index(1);
			}
		}
	}
	if (label_white.Is())
		label_white.Set_Text(ICS::Get_White());
	if (label_black.Is())
		label_black.Set_Text(ICS::Get_Black());
	if (status_bar.Is(1))
		status_bar.Set(1, "New game");
	Timer::Start(1000);
	board().Clear_Selection();
	Update_Position();
}

void
ICS_Window::ICS_Finished()
{
	if (status_bar.Is(0))
	{
		int r= ICS::Get_Result();
		status_bar.Set
			(0,
			 Trl(r == 0 ? "Black wins" :
			     r == 1 ? "Draw"       :
			     r == 2 ? "White wins" :
			     "Aborted"));
	}
	if (Timer::Is())
		Timer::Stop();
	Update_Position();
}

void
ICS_Window::ICS_Playing(String move)
{
	Position p;
	p= ICS::Get_Position();
	Generator generator;
	generator.New(p);
	int sm= Parse_Move(move(), generator);
	if (-1 != sm)
	{
		Move m= generator.Get_Move(sm);
		board().Set_Selection(m.from,
				      m.to);
		move= san_notation.Format_Move_Name(generator.Get_Move_Name(sm));
	}
	else
		board().Clear_Selection();
	if (status_bar.Is(1))
		status_bar.Set(1, move);
	Update_Position();
}

void
ICS_Window::ICS_Message(String message)
{
  if (interaction_window.Is())
    interaction_window.Append_Message(message);
}

void
ICS_Window::ICS_Clock(int white,
		      int black)
{
  Set_White_Clock(white);
  Set_Black_Clock(black);
}

void
ICS_Window::Timer_Alarm()
{
	if (ICS::Get_Position().Get_To_Move() == cc_w)
	{
	  Set_White_Clock(- white_clock.Get() - 1);
	}
	else
	{
	  Set_Black_Clock(- black_clock.Get() - 1);
	}
}

void
ICS_Window::IR_Input()
{
  MR_Send(msg_input);
}

String ICS_Window::Format_Clock(int seconds)
{
  int minutes= seconds / 60;
  int hours= minutes / 60;
  seconds %= 60;

  if (hours)
    {
      return Format("%d:%c%c:%c%c", 
		    hours, 
		    minutes / 10 + '0', 
		    minutes % 10 + '0',
		    seconds / 10 + '0',
		    seconds % 10 + '0');
    }
  else
    {
      return Format("%d:%c%c", 
		    minutes, 
		    seconds / 10 + '0',
		    seconds % 10 + '0'); 
    }
}

void ICS_Window::Set_White_Clock(int seconds)
{
	white_clock.Set(- seconds);
	white_text_clock.Set_Text(Format_Clock(seconds));
}

void ICS_Window::Set_Black_Clock(int seconds)
{
	black_clock.Set(- seconds);
	black_text_clock.Set_Text(Format_Clock(seconds));
}

void ICS_Window::Update_Position()
{
  if (legal_move_list.Is())
    {
      Update_Legal_Moves();
    }

  if (ICS::Is() && ICS::Get_Inside_Game()  &&
      material_widget.Get_Show(ICS::Get_Position()))
    {
      material_widget.Set(ICS::Get_Position(),
			  ICS::Get_Playing_Side()); 
      material_widget.Set_Visible(true);
    }
  else
    {
      material_widget.Set_Visible(false);
    }

  bool have_promotion= ICS::Get_Generator().Is() && (ICS::Get_Generator().Get_Flags() & gn_promotion); 
  if (have_promotion)
	{
	  if (promotion_chooser.Is()  ||
	      promotion_chooser.Open(side_packer, persistent_notation()))
	    promotion_chooser.Set_Used(true);
	}
	else
	{
		if (promotion_chooser.Is())
			promotion_chooser.Set_Used(false);
	}

}

void ICS_Window::Open_Interaction_Window()
{
	interaction_window.Open(NULL,
				this,
				msg_text);
}

void ICS_Window::User_Play(int move_index,
			   int from,
			   int to)
{
  if (from == -1)
    from= ICS::Get_Generator()[move_index].from;
  if (to == -1)
    to= ICS::Get_Generator()[move_index].to;

				if (! (ICS::Is() && ICS::User_Play(from, to)))
				{
				}
				else
				{
					status_bar.Set
						(1,
						 san_notation.Format_Move_Name
						 (ICS::Get_Generator().Get_Move_Name(move_index)));
					board().Set_Selection(from, to);
					Update_Position();
				}
}

void ICS_Window::Update_Legal_Moves()
{
  if (! ICS::Get_Generator().Is())
    return; 
      legal_move_list.Set(ICS::Get_Generator(),
			  persistent_notation());
}

