#ifndef ICS_WINDOW_INCLUDED
#define ICS_WINDOW_INCLUDED

#include "buffer.hh"
#include "bevel.hh"
#include "status_bar.hh"
#include "ics.hh"
#include "clean_board.hh"
#include "ics_account.hh"
#include "ics_account_editor.hh"
#include "clock.hh"
#include "timer.hh"
#include "input_reader.hh"
#include "menu_window.hh"
#include "ics_interaction_window.hh"
#include "legal_move_list.hh"
#include "notation_editor.hh"
#include "persistent.hh"
#include "notation_stream.hh"
#include "material_widget.hh"
#include "promotion_chooser.hh"

class ICS_Window
        :  public V_Packer,
	   private ICS,
	   public Timer,
	   public Input_Reader,
	   public Menu_Window
{
public:

	typedef String Open_Info;

	bool Open(Pointer <Widget> , Open_Info);
	static const char * Get_Help()
		{
			return "Play chess over the net.\n";
		}

private:

	enum {msg_board= V_Packer::msg_custom, /* msg_input, */ 
	      msg_account_edited,
	      msg_login, msg_logout, msg_seek,
	      msg_resign, msg_text, 
	      /* msg_scroll_end, */  msg_about, msg_legal_move_list,
	      msg_notation, msg_input, msg_interaction, msg_show_legal, 

	      menu_main= 0, menu_file, menu_help, menu_game, menu_view};

	H_Packer top_packer;
	V_Packer side_packer;
	H_Packer black_packer, white_packer;
	Text_Widget label_white, label_black;
	Clock white_clock, black_clock;
	V_Packer white_clock_packer, black_clock_packer;
	Text_Widget white_text_clock, black_text_clock; 
	Clean_Board board;
	Status_Bar status_bar;
	ICS_Account ics_account;
	Owned_Editor <ICS_Account_Editor> account_editor;
	ICS_Interaction_Window interaction_window;
	Legal_Move_List legal_move_list; 
	/* Always visible if open. 
	 */
	Material_Widget material_widget;
	Promotion_Chooser promotion_chooser; 

	Persistent <Notation, Notation_Editor> persistent_notation;

	void Wdg_Close();
	String Wdg_Init_Caption() const
		{
			return Trl("Internet Chess Server");
		}
	
	void MR_Receive(int);

	void Menued_Get(int menu, Menued::Entries &entries);

	void ICS_Changed();
	void ICS_New_Game();
	void ICS_Finished();
	void ICS_Playing(String);
	void ICS_Message(String _message);
	void ICS_Clock(int, int);

	void Timer_Alarm();
	void IR_Input();

	void Moved();
	void Seek(int, int);
	void Connect();
	void Logout();
	void Set_White_Clock(int seconds);
	void Set_Black_Clock(int seconds);
	void Update_Position();
	void Open_Interaction_Window();
	void User_Play(int move_index,
		       int from= -1,
		       int to= -1);
	void Update_Legal_Moves();

	static String Format_Clock(int seconds);
};

#endif /* ! ICS_WINDOW_INCLUDED */
