#include "main.hh"

/* Macro set in `macro':

       MAIN_NO_PROGRAM_NAME:  Assume the variable
              `program_invocation_name' does not exist.  If it exists,
	      it is in <errno.h>. 
*/

#include "dbg.hh"

#include <unistd.h>

void (*main_hook)(int *argc, char ***argv)= NULL;

#ifdef MAIN_NO_PROGRAM_NAME
const char *program_invocation_name= NULL;
#endif

int
Main(int argc, char **argv);

int
main(int argc, 
     char **argv)
{
	Dbg("entering main() in PID= %d.",
	    (int)getpid());

#ifdef MAIN_NO_PROGRAM_NAME
	program_invocation_name= argv[0];
#endif

	if (main_hook != NULL)
		main_hook(& argc, & argv);

	int result= Main(argc, argv);

	Dbg("quitting main() in PID= %d.",
	      (int)getpid());

	return result;
}

#if (! defined (NDEBUG)) && ! defined (MAIN_NO_SIGNAL)

#include <signal.h>

class Main_Signal
{
public:
  
	Main_Signal()
		{
			const int count= 9;
			int sigs[count]= 
			{SIGTERM, SIGHUP, SIGPIPE, SIGINT, SIGQUIT,
			 SIGSEGV, SIGBUS, SIGFPE, SIGILL};

			for (int i= 0;  i < count;  ++i)
				signal(sigs[i], 
				       & Handler);
		}

private:

	static void Handler(int signal)
		{
			Dbg_Function();
			Dbg_Int(signal);
			if (signal == SIGTERM ||
			    signal == SIGHUP  ||
			    signal == SIGPIPE ||
			    signal == SIGINT  ||
			    signal == SIGQUIT)
				abort();
			else
				Assert_Unreached();
		}

} main_signal;

#endif /* ! NDEBUG && ! SIGNAL */

void
Main_Set_Hook(void (*function)(int *argc, char ***argv))
{
	Assert (main_hook == NULL);
	main_hook= function;
}


