#ifndef MATERIAL_INCLUDED
#define MATERIAL_INCLUDED

/* The count of pieces for each side of a position.  The number of
   kings is also counted.  Counting the king count is easier and more
   efficient than checking for "pt != pt_K" everytime. 
 */

class Material
{
public:

	int operator () (int pt, int cc) const
		{
			Assert (pt >= 0 && pt < 6 &&
				cc >= 0 && cc < 2);
			return count[pt][cc];
		}
	int &operator () (int pt, int cc)
		{
			Assert (pt >= 0 && pt < 6 &&
				cc >= 0 && cc < 2);
			return count[pt][cc];
		}

	void Clear()
		{
		  memset(count, '\0', sizeof (count));
		}

#ifndef NDEBUG

	bool Check() const
		{
			for (int pt= 0;  pt < 6;  ++pt)
				for (int cc= 0;  cc < 2;  ++cc)
					if (count[pt][cc] < 0)
						return false;
			return true;
		}

#endif /* ! NDEBUG */

private:

	int count[6][2];
};

#endif /* ! MATERIAL_INCLUDED */


