#include "mem.hh"

/* malloc() may return NULL or allocate a chunk of minimal
   size. 
   
   If size is NULL, realloc() may return NULL or shrink the
   chunk to the smallest possible size. 
*/

//#ifdef MEM_DEFAULT

#ifndef NDEBUG

/* Debugging versions. 
*/

#include "assert.hh"
#include "constant.hh"
//#include "dbg.hh"
#include "debug.hh"
//#include "env.hh"

#include <stdlib.h>
#include <string.h>

//#ifdef MEM_DBG


#define Mem_Dbg(args...) if (getenv("MEM_DBG")) Debug(args)

int Mem::count= 0;

//#endif /* MEM_DBG */

void *
Mem::Alloc(int size)
{
	Assert (size >= 0);
	Assert (size < 0x8000000);

	void *result;

	if (size == 0)
	{
		result= NULL;
	}
	else
	{
		result= malloc(size);
		Assert (result == NULL || ! Constant(result));
	}

//#ifdef MEM_DBG

//	if (size != 0 && result == NULL)
//	  Debug("malloc() failed.");

	if (result != NULL)
		++ count;

	Mem_Dbg("Alloc    (            %8d)\t[%6d]",
	    size,
	    count);
	
//#endif /* MEM_DBG */

	return result;
}

void *
Mem::Re_Alloc(void *pointer, 
	      int size)
{
	Assert (size >= 0);
	Assert (size < 0x8000000);

	void *result= NULL;
	if (size == 0)
	{
		free(pointer);
		result= NULL;
	}
	else
	{
		result= realloc(pointer, size);
		Assert (result == NULL || ! Constant(result));
	}

//#ifdef MEM_DBG

//	if (size != 0 && result == NULL)
//		Debug("No memory!");

	if (pointer == NULL && size != 0)
		++ count;
	else if (pointer != NULL && size == 0)
		-- count;

	Mem_Dbg("Re_Alloc (%10p, %8d)\t[%6d]",
	      pointer,
	      size,
	      count);

//#endif /* MEM_DBG */

	return result;
}

void
Mem::Free(void *pointer)
{
//#ifdef MEM_DBG

	if (pointer != NULL)
		-- count;

	Mem_Dbg("Free     (%10p          )\t[%6d]",
	      pointer,
	      count);

//#endif /* MEM_DBG */

	free(pointer);
}

//#endif /* ! MEM_NDEBUG */

#endif /* MEM_DEFAULT */
