#include "menu_item.hh"

#include "dbg.hh"

void
Menu_Item::Activate()
{
	Dbg_Function();

	if (! popup_menu.Is())
		/* menu item is command. */ 
	{
		Dbg("\tSending msg= %d", id);
		if (menued.Is()) 
			/* If MENUED is NULL, the item may be a separator. 
			 */
			menued().MR_Send(id);

		return;
	}

	/* Open submenu. 
	 */

	if (! popup_menu().Is())
		return; 

	Menued::Entries entries;
  
	entries.Set_Menued(menued()); 

	menued().Menued_Get(id, entries); 

	popup_menu().Clear(); 
	popup_menu().Append(entries); 
}

bool
Menu_Item::Open(Pointer <Widget> parent,
		const Menued::Entry &entry)
{
	Dbg_Function();
	Dbg("\ttext= %s", entry.text());
	Dbg("\tid=   %d", entry.id);

	String text= entry.text;
#if 0
	if (entry.type != Menued::Entry::type_separator)
	{
		if (entry.glyph_type == gl_radio)
			text= "(*) " + entry.text;
	}
#endif /* 0 */ 
	GtkWidget *widget;
	if (entry.type == Menued::Entry::type_separator)
		widget= gtk_menu_item_new();
	else
	{
		switch (entry.glyph_type)
		{
		default:  
			widget= gtk_menu_item_new_with_label(text()); 
			break;

		case gl_check:
			widget= gtk_check_menu_item_new_with_label(text());
			if (widget == NULL)
				return false;
			gtk_check_menu_item_set_active
				(GTK_CHECK_MENU_ITEM(widget), TRUE);
			gtk_check_menu_item_set_show_toggle
				(GTK_CHECK_MENU_ITEM(widget), TRUE);
			break;

		case gl_radio:
			widget= gtk_radio_menu_item_new_with_label(NULL, text());
			if (widget == NULL)
				return false;
			gtk_check_menu_item_set_active
				(GTK_CHECK_MENU_ITEM(widget), TRUE);
			gtk_check_menu_item_set_show_toggle
				(GTK_CHECK_MENU_ITEM(widget), TRUE);
			break;
		}
	}

	if (! Widget::Open(parent, widget))
		return false;

	if (! entry.enabled)
		gtk_widget_set_sensitive
			(Get(),
			 FALSE);

	id= entry.id; 
	menued= entry.target; 

	gtk_signal_connect(GTK_OBJECT(Get()), "activate",
			   GTK_SIGNAL_FUNC(Signal_Activate), this); 

	if (entry.type == Menued::Entry::type_sub)
	{
		if (popup_menu.Alloc())
		{
			if (popup_menu().Open(NULL))
			{
				gtk_menu_item_set_submenu(GTK_MENU_ITEM(Get()),
							  popup_menu().Get()); 
			}
		}
	}

	return true; 
}

void
Menu_Item::Wdg_Close()
{
	menued= NULL;
	if (popup_menu.Is())
	{
		if (popup_menu().Is())
			popup_menu().Close(true); 
		popup_menu.Free(); 
	}
	Widget::Wdg_Close(); 
}

void Gtk_Popup_Menu::Append(const Menued::Entries &entries)
{
	for (int i= 0;  i < *entries;  ++i)
	{
		Menued::Entry entry= entries[i];

		if (! items.Set(*items + 1))
			return; 

		if (! items[-1].Alloc())
		{
			items= -1;
			return; 
		}

		if (! items[-1]().Open(This,
				       entry))
		{
			items= -1;
			return;
		}
	}
}

void Gtk_Popup_Menu::Clear()
{
	for (int i= 0;  i < *items;  ++i)
		items[i]().Close(true);
	items.Free(); 
}

Gtk_Popup_Menu::~Gtk_Popup_Menu()
{
}

