#ifndef MULTI_LIST_BOX_INCLUDED
#define MULTI_LIST_BOX_INCLUDED

#include "widget.hh"
#include "scroll_direction.hh"
#include "gtk_scrolled.hh"

class Multi_List_Box_Content
/* Not scrolled. 
 */
	:  /*virtual*/ public Widget
{
public:

	class Open_Info
	{
	public: 

		int column_count;
		Scroll_Direction scroll_direction;
		int msg_clicked;
		Pointer <const Vector <String> > titles;

		Open_Info(int new_column_count= 2,
			  Scroll_Direction new_scroll_direction= scr_both,
			  int new_msg_clicked= msg_ignore,
			  Pointer <const Vector <String> > new_titles= NULL)
			  
			:  column_count(new_column_count),
			   scroll_direction(new_scroll_direction),
			   msg_clicked(new_msg_clicked),
			   titles(new_titles)
			{
			}
	}; 

	bool Open(Pointer <Widget> parent,
		  int new_column_count= 2,
		  Scroll_Direction scr= scr_both,
		  int msg_clicked= msg_ignore,
		  Pointer <const Vector <String> > titles= NULL); 
	bool Open(Pointer <Widget> parent,
		  Open_Info open_info)
		{
			return Open(parent,
				    open_info.column_count,
				    open_info.scroll_direction,
				    open_info.msg_clicked,
				    open_info.titles); 
		}

	void Get_Clicked(int &r, int &c)
		{
			r= row;
			c= column;
		}
	void Set_Selected(int new_row);

	int Get_Selected() const
		{
			return row;
		}

	void Set(int column, int row, String text)
		{
			Dbg_Function();
			Dbg_Int(column);
			Dbg_Int(row);
			Dbg_Str(text());
			gtk_clist_set_text(GTK_CLIST(Get()),
					   row, column, text());
		}

	void Set_Row_Count(int rows);
	int Get_Row_Count() const
		{
			return row_count;
//			return next_column == 0 ? next_row : next_row + 1;
		}
	void Set_Title(int col, String title)
		{
			gtk_clist_set_column_title
				(GTK_CLIST(Get()),
				 col,
				 title());
		}

	bool Get_Double_Click() const
		{
			return double_click; 
		}

private:

//	int next_column, next_row; 

	int row_count;

	int column_count;
	/* Set in Open(), never changed. 
	 */

	/* Set by Signal_Select_Row().  Returned by Get_Clicked(). 
	 */
	int row, column;

	int msg_clicked; 
//	int max_width;
	bool double_click;

	void Wdg_Get_Size_Request(int & /* width */ ,
				  int &height,
				  bool &resizeable)
		{
			height= 4 * unit; 
			resizeable= false; 
		}

	static void Signal_Select_Row (GtkCList *clist,
				       gint row,
				       gint column,
				       GdkEventButton *event,
				       gpointer user_data);
};

class Multi_List_Box
	:  public Gtk_Scrolled <Multi_List_Box_Content> 
{
public:

	typedef Multi_List_Box_Content::Open_Info Open_Info;

	bool Open(Pointer <Widget> parent,
		  Open_Info open_info);

	void Set_Selected(int row, bool scroll_to_visible= false);

	int Get_Selected() const
		{
			return This().Get_Selected(); 
		}
	void Get_Clicked(int &r, int &c)
		{
			This().Get_Clicked(r, c); 
		}
	void Set(int column, int row, String text)
		{
			This().Set(column, row, text);
		}
	void Set_Row_Count(int rows)
		{
			This().Set_Row_Count(rows);
		}
	int Get_Row_Count() const
		{
			return This().Get_Row_Count(); 
		}
	void Set_Title(int col, String title)
		{
			This().Set_Title(col, title);
		}
};

#endif /* ! MULTI_LIST_BOX_INCLUDED */ 
