#include "notation.hh"

#include "dbg.hh"

String Notation::
Format_Move_Name(Move_Name move_name)
	const
{
	Dbg_Function();
	Dbg_Int(move_name.from_file);
	Dbg_Int(move_name.to_file);

	String result;

	switch (move_name.special)
	{
	case 2:
		result= "O-O";
		break;

	case 3:
		result= "O-O-O";
		break;

	default:

		Append_Sign(result, move_name.piece_type);

		if (! (options & nt_short))
			/* Long notation */ 
		{
			result += Format_Square(Make_Square(move_name.from_file, move_name.from_rank));

			if (move_name.capture == 0)
				result += '-';
			else
			{
				if (capture != '\0')
					result += capture; 
			}

			if (move_name.capture != 0 && (options & nt_capture_name))
				Append_Sign(result, move_name.capture - 1);
			result += Format_Square(Make_Square(move_name.to_file, move_name.to_rank));
		}
		else
			/* Short notation */ 
		{
			if (move_name.piece_type != pt_P)
			{
				if (move_name.precision % 2)
					result += 'a' + move_name.from_file;
				if (move_name.precision / 2)
					result += '1' + move_name.from_rank;
				if (move_name.capture != 0 && 
				    capture != '\0'        &&
				    (options & nt_short_capture) &&
				    (! (options & nt_short_capture_end)))
					result += capture;
			}
			else
			{
				if (move_name.capture != 0)
				{
					result += 'a' + move_name.from_file;
					if ((options & nt_short_pawn) && 
					    capture != '\0'           &&
					    (options & nt_short_capture) &&
					    ! (options & nt_short_capture_end))
						result += capture;
				}
			}

			result += Format_Square(Make_Square(move_name.to_file, move_name.to_rank));

			if (move_name.capture != 0 &&
			    capture != '\0'        &&
			    (options & nt_short_capture) && 
			    (options & nt_short_capture_end) &&
			    (move_name.piece_type != pt_P || (options & nt_short_pawn)))
				result += capture;
		}

		if ((options & nt_ep) && move_name.special == 1)
			result += " ep";
		if (move_name.special >= 4)
		{
			if (promotion != '\0')
				result += promotion;
			Append_Sign(result, move_name.special - 3);
		}
	}

	if (options & nt_check)
		switch (move_name.check)
		{
		default:  break;
		case 1: result += '+'; break;
		case 3: result += '#'; break;
		}

	Dbg_Str(result()); 

	return result;
}


const Piece_Type san_chars[26]=
   {8, 2, 1, 4, 8, 2, 8, 8, 8, 8, 5, 2, 8, 1, 8, 0, 4, 3, 1, 3, 8, 8,
 /* A  B  C  D  E  F  G  H  I  J  K  L  M  N  O  P  Q  R  S  T  U  V
  */
    8, 8, 8, 8};
 /* W, X, Y, Z
  */

Notation san_notation
("PNBRQK",
 'x',
 '=',
 nt_short | nt_check | nt_short_capture | nt_short_pawn); 

void Notation::Reset()
{
	This= san_notation; 
}

