#include "radio_chooser_content.hh"

#include "text_radio_button.hh"

bool
Radio_Chooser_Content::Append(String text)
{
	Assert (Is());

	if (! buttons.Set(*buttons + 1))
		return false;

	Dynamic <Text_Radio_Button> text_radio_button;
	if (! text_radio_button.Alloc())
	{
		buttons= -1;
		return false;
	}

	if (! text_radio_button().Open
	    (this, 
	     text,
	     msg_clicked + *buttons - 1,
	     *buttons == 1 ? NULL : & buttons[0]()))
	{
		buttons= -1;
		return false;
	}

	text_radio_button.Transfer_To(buttons[-1]);
	
	return true; 
}

bool
Radio_Chooser_Content::Append(Dynamic <Checkable_Widget> &widget)
{
	Assert (widget.Is()); 
	Assert (widget().Get_Parent() == this);
	
	if (! buttons.Set(*buttons + 1))
	{
		widget().Close();
		widget.Free(); 
		return false;
	}

	widget >> buttons[-1]; 
	
	return true; 
}

void 
Radio_Chooser_Content::MR_Receive(int msg)
{
	if (msg < msg_clicked)
		return Grid_Packer::MR_Receive(msg); 

	int index= msg - msg_clicked; 

	Assert (index >= 0 && index < *buttons); 

	if (selected == index)
		return;

	selected= index;
	Parent_Send_Message(msg_changed);
}

bool
Radio_Chooser_Content::Open(Pointer <Widget> parent, 
			    Open_Info open_info)
{
	Assert (open_info.columns >= 1);

	msg_changed= open_info.msg_changed;
//	selected= -1;
	selected= 0; 
	columns= open_info.columns;
	buttons.Free();

	return Grid_Packer::Open(parent, columns);
}

void Radio_Chooser_Content::Set_Selected(int new_selected)
{
	Assert (new_selected >= -1 && new_selected < *buttons);
	
	if (selected == new_selected)
		return;
	if (selected != -1)
		buttons[selected]().Set_Checked(false);
	selected= new_selected;
	if (selected != -1)
		buttons[selected]().Set_Checked(true);
}

void Radio_Chooser_Content::Clear()
{
	Assert (Is());

	for (int i= 0;  i < *buttons;  ++i)
		buttons[i]().Close();

	buttons= 0;
	selected= -1;
}
