#include "stat.hh"

/* Macros set in `macro':

   STAT_NO_LSTAT:  don't use lstat().  

 */

#include <sys/stat.h>
#include <fcntl.h>

bool 
Get_Stat(const Stream &stream,
	 Stat &stat)
{
	Assert (stream.Is()); 

	struct stat s;
	if (0 != fstat(stream.Get_FD(), &s))
		return false;
	return Make_Stat(stat, s);
}

bool
Get_Stat(String name,
	 Stat &stat,
	       bool 
#ifndef STAT_NO_LSTAT
	       follow
#endif
	       ) 
{
	struct stat s;
	int t= 

#ifdef STAT_NO_LSTAT

		stat(file_name, &s);

#else /* ! STAT_NO_LSTAT */

	(follow
	 ? ::stat(name(), &s) 
	 : lstat(name(), &s));

#endif /* ! STAT_NO_LSTAT */

	if (t)
		return false;

	return Make_Stat(stat, s); 
}

bool
Make_Stat(Stat &stat, 
		struct stat &s)
{
	mode_t m= s.st_mode;
	if (S_ISDIR(m))
		stat.type= stat_dir;
	else if (S_ISREG(m))
		stat.type= stat_regular;
	else if (S_ISFIFO(m))
		stat.type= stat_pipe;
	else if (S_ISLNK(m))
		stat.type= stat_link;
	else if (S_ISBLK(m))
		stat.type= stat_block;
	else if (S_ISCHR(m))
		stat.type= stat_character;
	else if (S_ISSOCK(m))
		stat.type= stat_socket;

	stat.size= s.st_size;

	stat.perm.user= 0;
	if (s.st_mode & S_IRUSR)
		stat.perm.user |= Stream::p_read;
	if (s.st_mode & S_IWUSR)
		stat.perm.user |= Stream::p_write;
	if (s.st_mode & S_IXUSR)
		stat.perm.user |= Stream::p_execute;
	if (s.st_mode & S_ISUID)
		stat.perm.user |= Stream::p_extra;
	stat.perm.group= 0;
	if (s.st_mode & S_IRGRP)
		stat.perm.group |= Stream::p_read;
	if (s.st_mode & S_IWGRP)
		stat.perm.group |= Stream::p_write;
	if (s.st_mode & S_IXGRP)
		stat.perm.group |= Stream::p_execute;
	if (s.st_mode & S_ISGID)
		stat.perm.group |= Stream::p_extra;
	stat.perm.other= 0;
	if (s.st_mode & S_IROTH)
		stat.perm.other |= Stream::p_read;
	if (s.st_mode & S_IWOTH)
		stat.perm.other |= Stream::p_write;
	if (s.st_mode & S_IXOTH)
		stat.perm.other |= Stream::p_execute;
	if (s.st_mode & S_ISVTX)
		stat.perm.other |= Stream::p_extra;

	stat.a_date= Make_Date(s.st_atime);
	stat.m_date= Make_Date(s.st_mtime);
	stat.c_date= Make_Date(s.st_ctime);

	return true; 
}


