#include "stream.hh"

/* Macros:
   STREAM_HAVE_UNLOCKED:  Use the `*_unlocked' versions of some
                        functions. 
*/

//#include "str_utils.hh"
#include "deref.hh"
#include "error.hh" 

void (*stream_hook)(int fd)= NULL;
/* Called for each FD created.  Set in Set_FD_Creation_Hook().  The
   PROCESS module registers a hook that sets each FD opened to
   close-on-exec.  
 */

void
Stream_Call_Hook(int fd)
{
	if (stream_hook != NULL)
		stream_hook(fd);
}

bool
Stream::Open(int fd,
	   const char *mode)
{
	Assert (! Is());
	Assert (fd >= 0);

	FILE *fdo= fdopen(fd, mode);
	if (fdo == NULL)
	{
		return false;
	}
	file= fdo;

	Stream_Call_Hook(fileno(file));

	return true;
}

bool
Stream::Do_Open(String file_name,
	      int mode, 
	      int type)
{
	Assert (mode >= 0 && mode < mode_count);
	Assert (type >= 0 && type < type_count);
	char ms[3];
	char open_modes[]= {'r', 'w', 'a'};
	char types[]= {'t', 'b'};
	ms[0]= open_modes[mode];
	ms[1]= types[type];
	ms[3]= '\0';
	file= fopen(file_name(), ms);
	if (file == NULL)
	{
		Error::Set_File_Name(file_name); 
		return false;
	}

	Stream_Call_Hook(fileno(file));

	return true;
}

bool
Stream::Open(String file_name, 
	   int mo, 
	   int ty)
{
	Assert(! Is());

	return Do_Open(file_name, mo, ty);
}

bool
Stream::Remove(String file_name)
{
	Assert (file_name.Is()); 
	return 0 == remove (file_name());
}

bool
Stream::Dup(int f, 
	  const char *mod)
{
	Assert (! Is());
  
	int d= dup(f);

	if (d == -1)
		return false;

	FILE *fdo= fdopen(d, mod);
	if (fdo == NULL)
	{
		close(d);
		return false;
	}
	file= fdo;

	Stream_Call_Hook(fileno(file));

	return true;
}

Stream::~Stream()
{
	if (type == -1           &&
	    file != NULL)
		fclose(file);
	/* Don't close stdin & co.
	 */
}

void
Stream::Set_Block(bool _block)
{
	Assert (Is());

	int _flags= fcntl(Get_FD(),
			  F_GETFL);
	if (_block)
		_flags &= (~ O_NONBLOCK);
	else
		_flags |= O_NONBLOCK;

	fcntl(Get_FD(),
	      F_SETFL,
	      _flags);
}

Stream::Stream(int fd)
{
	type= 1 + fd;
	file= NULL;
}

void
Stream::Open_FD(int fd)
{
	Assert (! Is());

	type= 1 + fd;
}

void
Stream::Set_FD_Creation_Hook(void (*f)(int fd))
{
	Assert (stream_hook == NULL);
	stream_hook= f;
}
