#include "main.hh"
#include "hash_table.hh"
#include "random.hh" 
#include "time.hh" 
#include "io.hh" 
#include "format.hh"
#include "write_formatted.hh"
#include "dbg.hh"

class Printer
	:  public Hash_Table::Callback
{
	void Call(String key,
		  String value)
		{
			Dbg("\"%s\"  \"%s\"", key(), value());
		}
};

Inline
char Get_Random_Char()
{
	char result= 'a' + Get_Random() % 26; 

	return result; 
}

int 
Main(int, char **)
{
	Dbg_Function();
//	Dbg("\tBeginning test...");

	Write_Formatted(IO::out, "beginning test...\n");

	Time begin_time= Get_Time(); 

	Hash_Table hash_table; 

	for (int i= 0;  i < 10000;  ++i)
	{
		const int length= 10; 

		char string[length + 1];

		for (int j= 0;  j < length;  ++j)
			string[j]= Get_Random_Char(); 

		string[length]= '\0'; 

		String key(string); 
		String val= Format("Number %d", i);
		Dbg("\t[%7d]  add  \"%s\"  \"%s\"",
		    i, key(), val());
		hash_table.Add(key, val);
	}

	Time end_time= Get_Time(); 

	int ms= Time_To_ms(end_time - begin_time); 

	Write_Formatted(IO::out, "fill time:  %d ms\n",
			ms);

	Printer printer;

	hash_table.Iterate(printer);

	hash_table.Add("key", "Guppi");
	
	String value;
	if (hash_table.Get("key", value))
		Dbg_Str(value());
	else
	{
		Write_Formatted(IO::err, 
				"***");
		return 1;
	}

	hash_table.Remove("key");
	if (hash_table.Get("key", value))
	{
		Write_Formatted(IO::err, 
				"***");
		return 1;
	}

	return 0; 
}
