#include "widget.hh"

#include "dbg.hh"

#include <gdk/gdk.h>

int Widget::depth= -1; 

bool
Widget::Open(Pointer <Widget> parent) 
{
	GtkWidget *gtkwidget= gtk_drawing_area_new(); 

	if (gtkwidget == NULL)
		return false;

	Connect_Mouse_Signals(gtkwidget);

	if (!  Open(parent, gtkwidget))
		return false;

	int w= -1, h= -1;
	bool resizeable= false; 
	Wdg_Get_Size_Request(w, h, resizeable); 
  
	Dbg_("Result of Wdg_Get_Size_Request():");
	Dbg_("\tw=      %d", w);
	Dbg_("\th=      %d", h);
	Dbg_("\tresizable= %s", resizeable ? "true" : "false");

	if (w >= 0 && h >= 0)
	{
		if (! resizeable)
		{
			gtk_drawing_area_size(GTK_DRAWING_AREA(gtkwidget),
					      w, h);
		}
		else
		{
		}
	}
  
	return true;
}

bool
Widget::Open(Pointer <Widget> new_parent,
	     GtkWidget *new_gtkwidget,
	     bool open_window)
{
	Assert (! Is()); 

	if (new_gtkwidget == NULL)
		return false;
	if (depth == -1)
	{
		GdkVisual *visual=
			gtk_widget_get_visual(new_gtkwidget);
		if (visual != NULL)
		{
			depth= Deref(visual).depth; 
		}
	}
	Gtk_Object::Attach(new_gtkwidget);
	if (new_parent.Is())
	{
		parent= new_parent(); 

		if (! parent().Append(this))
		{
			Gtk_Object::Close();
			return false;
		}
		gtk_widget_show(new_gtkwidget); 

		parent().Wdg_Append_Child(This); 
	}
	else
	{
		if (open_window)
		{
			Point window_size= Wdg_Get_Window_Size();
			if (! gtk_window.Open(Wdg_Init_Caption(), window_size))
			{
				Gtk_Object::Close();
				return false;
			}

			gtk_window.Set(This);
		}
	}

	gtk_widget_show(new_gtkwidget);

	return true;
}

bool
Widget::Append(Pointer <Widget> child)
{
	if (! children.Set(*children + 1))
		return false;
	children[-1]= child;
	return true; 
}

void
Widget::Wdg_Do_Close()
{
	Gtk_Object::Wdg_Do_Close(); 

	if (gtk_window.Is())
	{
		gtk_widget_destroy(gtk_window.Get()); 
	}

	Assert (parent == NULL);
	Assert (*children == 0); 
}

void
Widget::Wdg_Destroy()
{
}

void
Widget::Wdg_Close()
{
	children.Free();
	if (parent.Is())
	{
		for (int i= 0;  i < *parent().children;  ++i)
		{
			if (parent().children[i] == this)
			{
				if (i + 1 < *parent().children)
					parent().children[i]= parent().children[-1];
				parent().children= -1; 
			}
		}
		parent= NULL; 
	}
	Message_Receiver::Remove_Pending_Messages();

	Gtk_Object::Wdg_Close(); 
}

Widget::Widget()
{
	Dbg_Function();
	Dbg_Pointer(this);
}
#if 0
static bool gtk_widget_idle_is= false;
static guint gtk_widget_idle_id;

void
Widget::MR_Send(int msg)
{
	Dbg_Function();

	if (! gtk_widget_idle_is)
	{
		gtk_widget_idle_is= true;
		gtk_widget_idle_id=
			gtk_idle_add_priority
			(GTK_PRIORITY_HIGH,
			 & Widget::Idle, 
			 NULL);
	}

	if (! messages.Set(*messages + 1))
		return;

	messages[-1].widget= this;
	messages[-1].msg= msg;
}

gint Widget::Idle(gpointer)
{
	Dbg_Function();

	Assert (gtk_widget_idle_is);
	Assert (*messages);

	if (messages[0].widget != NULL)
		messages[0].widget().MR_Receive(messages[0].msg);
	for (int i= 0;  i + 1 < *messages;  ++i)
		messages[i]= messages[i + 1];
	messages = -1;
	if (! *messages)
	{
		gtk_widget_idle_is= false;
		gtk_idle_remove(gtk_widget_idle_id);
	}
	return TRUE;

#if 0
	for (int i= 0;  i < *messages;  ++i)
	{
		if (messages[i].widget != NULL)
			messages[i].widget().MR_Receive(messages[i].msg);

		if (gtk_events_pending()) 
		{
			for (int j= i;  j < *messages;  ++j)
				messages[j - i]= messages[j];
			messages= i;
			return TRUE;
		}
	}

	messages= 0;

	gtk_widget_idle_is= false;
	gtk_idle_remove(gtk_widget_idle_id);

	return TRUE;
#endif /* 0 */ 
}
#endif /* 0  */ 
void
Widget::Set_Mouse_Cursor(Mouse_Cursor /* mc */ )
{
#if 0
	Assert (mc >= 0 && mc < ms_count);

	GdkWindow *w= gtk_widget_get_parent_window(Get());

	if (w == NULL)
		return;

	static GdkCursor *cursors[ms_count]=
		{NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
  
	if (cursors[mc] == NULL)
	{
		GdkCursorType val[ms_count]=
			{GDK_ARROW, GDK_CROSS, GDK_LEFT_SIDE, GDK_TOP_SIDE,
			 GDK_BOTTOM_LEFT_CORNER, GDK_BOTTOM_RIGHT_CORNER,
			 GDK_HAND1, GDK_GUMBY};

		cursors[mc]= gdk_cursor_new(val[mc]);
		if (cursors[mc] == NULL)
			return;
	}

	gdk_window_set_cursor(w, cursors[mc]);
#endif /* 0 */ 
}

void Widget::MR_Receive(int msg)
{
	if (Get_Parent().Is())
	{
		Get_Parent()().MR_Receive(msg); 
		return;
	}

	switch (msg)
	{
	case msg_ok:
	case msg_cancel:
		Close(); 
		break; 
	}
}

Window &Widget::Get_Window()
{
	if (gtk_window.Is())
		return gtk_window;
	else
		return parent().Get_Window(); 
}

void Widget::Wdg_Mouse(Mouse_Action, Mouse_Button, Point,
		       bool)
{
}
#if 0
void Widget::Set_Selection(String)
{
}
#endif /* 0 */ 
void Widget::Set_Transient(const Widget &widget)
{
	Assert (Is());
	Assert (widget.Is()); 

	Set_Transient(GTK_WINDOW(Get_Gtk_Window()),
		      GTK_WINDOW(widget.Get_Gtk_Window()));
}

Widget::~Widget()
{
	Dbg_Function();
	Dbg_Pointer(this);
}

void Widget::Wdg_Append_Child(Widget &widget)
{
	gtk_container_add(GTK_CONTAINER(Get()),
			  widget.Get()); 
}

void Widget::Set_Visible(bool visible)
{
//  GtkWidget *widget= gtk_window.Is() ? gtk_window.Get() : Get();

	if (visible)
	{
		gtk_widget_show(/*widget*/Get());
	}
	else
	{
		gtk_widget_hide(/*widget*/Get());
	}
}
